#include "GameStateModule.H"
#include "ShutDown.H"
#include "SDLinit.H"
#include "Game.H"
#include "GameObj.H"
#include "Vector.H"
#include "SType.H"

using namespace std;

bool quit = false;

GameStateModule *gsmP;

void dumpAllObjs()
{
    cout << "All Object Dump" << endl;
    Game &game = gsmP->get_game();	
    for (int n=0; n<=game.get_player_num(); ++n) {
        const Game::ObjCont &objs = game.get_objs(n);
        FORALL (objs, it) {
            const GameObj *gob = (*it)->get_GameObj();
            cout << gob->bp_name() << " ";
            cout << ScriptObj::get_obj_id( (ScriptObj*)gob ) << endl;
            Vector<string> names;
            gob->attr_names( names );
            for (vector<string>::iterator it=names.begin(); it!=names.end(); ++it) {
                cout << "  " << *it ;
                stype val = gob->get_val(*it);
                switch ( val.type ) {
                case stype::INT:
                    cout << " INT " << val.to_int() << endl; break;
                case stype::DIR:
                    // not sure what this is. a direction?
                    cout << " DIR" << endl; break;
                case stype::OBJ:
                    // pointer to a ScriptObj
                    cout << " OBJ " << ScriptObj::get_obj_id( val.to_obj() ) << endl; break;
                case stype::CSTR:
                    cout << " CSTR " << val.to_cstr() << endl; break;
                case stype::ADDR:
                    // pointer to another stype
                    cout << " ADDR" << endl; break;
                case stype::NONE:
                    cout << " NONE" <<endl; break;
                default:
                    cout << "Error: Unknown type" << endl;
                }
                //cout << gob
            }
        }
    }
}

class SampleEventHandler : public EventHandler {
public:
  SampleEventHandler() {}
  bool handle_event(const Event &e) { 
    if (e.get_who() == GameStateModule::FROM
		&& e.get_what() == GameStateModule::STOP_MSG)
      quit = true;
	if (e.get_who() == GameStateModule::FROM
		&& e.get_what() == GameStateModule::VIEW_MSG)
      dumpAllObjs();
      return true;
	}
};



int main(int argc, char **argv)
{
  signals_shut_down(true);  
  SDLinit::network_init();
  
  // create option objects (though we pass them no data)
  // and create modules

  GameStateModule::Options::add();
  Game::Options::add();  
  GameStateModule::Options gsmo;
  GameStateModule gsm(gsmo);
  
  gsmP = &gsm;
  
  SampleEventHandler* psev = new SampleEventHandler();

  // gsm events are handled in sev
  gsm.add_handler(psev);
  
  // connect game state module to server
  if (!gsm.connect()) ERR("connection problems");

	while ( ! quit )
		if (!gsm.recv_view()) SDL_Delay(1);

  return 0;
}
