// $Id: mapgen_main.C,v 1.1.1.1 2007/04/09 19:05:49 pjungkun Exp $

// This is an ORTS file (c) 2005 Michael Buro

#include "Mapgen.H"
#include "Global.H"
#include "Options.H"
#include "MapTool.H"

#include "Terrain.H"

using namespace std;

const char *ORTS_VERSION = "ORTS version 3.2 (2005-Nov-2)";

static void add_options()
{
  Options o("mapgen");
  o.put("-w3", string(""), "convert warcraft map into ORTS format");
  o.put("-mapfile", string("map.ortsm"), "name of the file to save the generated map to");
  o.put("-omin", sint4 (20), "minimum obstacle size");
  o.put("-omax", sint4 (50), "maximum obstacle size");
  o.put("-onum", sint4 (50), "number of obstacle to create");
  o.put("-margin", sint4 (25), "how far from the edge of the map base camps are placed");
  o.put("-base-size", sint4 (50), "size of a player's base camp");
  o.put("-center", double (0.5), "ratio determining how much the center is favored by when generating obstacle locations (1=favor center)");  
}

void write_file(string file, const stringstream & world)
{
  ofstream outStream;
  outStream.open(file.c_str());

  if(!outStream.is_open()) {
    cout << "Error opening output file" << endl;
    return;
  }

  outStream << world.str();
  outStream.close();

}

void write_world(stringstream & world, stringstream & objects, Map < Tile > &map_data)
{

  sint4 num_players = MapTool::opt.nplayers;
  sint4 tiles_x = MapTool::opt.width, tiles_y = MapTool::opt.height;
  sint4 z_sight = 0;

  world << num_players << endl;
  world << tiles_x << endl;
  world << tiles_y << endl;
  world << z_sight << endl;
  world << 0 << endl;

  world << "<TILES>" << endl;
  map_data.write(world);
  world << "</TILES>" << endl;

  string fname = MapTool::opt.bp_file;
  fname = sanitize_path(fname);
  MapTool::parse_map_include(world, fname);

  world << "<OBJECTS>\n";
  world << objects.str();
  world << "</OBJECTS>\n";
}


int main(int argc, char **argv)
{
  // test

  Terrain ter;

  ///////////
  
  add_options();
  MapTool::Options::add();

  if(Options::process(argc, argv, cerr, ORTS_VERSION))
    exit(20);

  MapTool::opt.get();

  stringstream world;

  string w3;
  ::Options::get("-w3", w3);

  if(w3 != "") {

    // transform warcraft map into ORTS representation

    exit(0);
  }

  sint4 tiles_x = MapTool::opt.width, tiles_y = MapTool::opt.height;
  cout << "tiles_x " << tiles_x << " tiles_y " << tiles_y << endl;
  Map < Tile > map_data;
  map_data.allocate(tiles_x, tiles_y);

  Random rand;
  rand.srand(MapTool::opt.seed);

  stringstream objects;

  sint4 margin; Options::get("-margin", margin);
  sint4 omin; Options::get("-omin", omin);
  sint4 omax; Options::get("-omax", omax);
  sint4 onum; Options::get("-onum", onum);
  sint4 base_size; Options::get("-base-size", base_size);
  double d; Options::get("-center", d);

  Mapgen mapgen;
  mapgen.init_map(map_data);
  mapgen.generate_obstacle_map(map_data, omin, omax, onum, d);
  mapgen.generate_basecamp_map(map_data, objects, MapTool::opt.nplayers, base_size, margin);
  mapgen.finalize_map(map_data);

  write_world(world, objects, map_data);

  string file;
  Options::get("-mapfile", file);
  write_file(file, world);

  cout << "DONE!" << endl;
  return 0;
}
