// $Id: ForceField.C,v 1.10 2006/07/26 21:21:24 

// This is not an ORTS file 

#include "ForceField.H"   
#include "GameStateModule.H"
#include "Game.H"
#include "GameChanges.H"
#include "GameObj.H"
#include "GfxModule.H"
#include "TerrainModule.H"
#include "Options.H"
#include <math.h>
#include <numeric>
#include <map>

#define PI 3.14159265
using namespace std;

uint4 myWorker = 0;

//***********************************************************************/
// Constants

static const int CC_FORCE = 500;
static const int CC_OFFSET = 48;
static const int MINERAL_FORCE = 15; 
static const int BOUNDARY_FORCE = 50;
static const int WORKER_FORCE = 30;
static const int SHEEP_FORCE = 50;
static const int WORKER_OFFSET = 5;
static const int SHEEP_OFFSET= 5;
static const int BOUNDARY_OFFSET= 10;
static const int INTER_CLUSTER_DIST = 23;
static const int WORKER_MINERAL_DISTANCE_THRESHOLD = 10;
/**********************************************************************/

ForceField::ForceField(GameStateModule &gsm_) : gsm(gsm_)
{
	Game &game = gsm.get_game();
	numMineralClusters = 0;
	ffMinerals = false;
	ffBoundary = false;
	plotFile.open("plot.txt", ios::out);
	debug.open("debug.txt", ios::out);
	myWorkerData.open("wData.txt", ios::out);
	sint4 tilePoints= game.get_tile_points();
	const Map<GameTile> &map = game.get_map();
	sint4 fieldWidth = map.get_width() * tilePoints;
	sint4 fieldHeight = map.get_height() * tilePoints;
	cout << "width: " << fieldWidth << ", height: " <<  fieldHeight << endl;
	allocate(fieldWidth, fieldHeight);
	
	
	Options::get("-bforce", boundaryForce);
	if (boundaryForce == -1) boundaryForce = BOUNDARY_FORCE;
	Options::get("-ccforce", ccForce );
	if (ccForce == -1) ccForce = CC_FORCE;
	Options::get("-mforce", mineralForce);
	if (mineralForce == -1) mineralForce = MINERAL_FORCE;
	Options::get("-wforce", workerForce);
	if (workerForce == -1) workerForce = WORKER_FORCE;
	Options::get("-sforce", sheepForce);
	if (sheepForce == -1) sheepForce = SHEEP_FORCE;
	Options::get("-cdist", interClusterDistance);
	if (interClusterDistance == -1) interClusterDistance = INTER_CLUSTER_DIST;
	Options::get("-ccrange", ccOffset);
	if (ccOffset == -1) ccOffset = CC_OFFSET;
	Options::get("-wrange", workerOffset);
	if (workerOffset == -1) workerOffset = WORKER_OFFSET;
	Options::get("-brange", boundaryOffset);
	if (boundaryOffset == -1) boundaryOffset = BOUNDARY_OFFSET;
	Options::get("-srange", sheepOffset);
	if (sheepOffset == -1) sheepOffset = SHEEP_OFFSET;
	Options::get("-nomc", noMineralClustering);
	
	string fname;
  Options::get("-outfile", fname);
    if( fname != "" )
      outFile.open(fname.c_str(),ios::app);
    else 
    	outFile.open("result.txt", ios::app);
}

//===================================================================

ForceField::~ForceField()
{
	delete [] staticFieldOutbound;
	delete [] dynamicFieldOutbound;
	// delete [] fieldOutbound;
}

//===================================================================

void ForceField::printField(const double *field, ScalarPoint pos, int offset)
{
		for (int y=pos.y - offset; y <= pos.y + offset; ++y) {
			for (int x=pos.x - offset; x <= pos.x + offset; ++x) {
			  //if (y == -1)
			  //	cout << x << " ";
			  //else
			  if (x >= 0 && y >= 0 && x < width && y <height)
				cout << field[y*width+x] << "\t";
				
			}
			cout << "\n" << endl;
		}
}

//===================================================================
void ForceField::printFieldGnuplot(const double *field)
{
		for (int y=0; y <= height; ++y) {
			for (int x=0; x <= width; ++x) {
			  //if (y == -1)
			  //	cout << x << " ";
			  //else
			  
				plotFile << x << " " << -y << " " << field[y*width+x] << endl;
				
			}
		}
}
//=================================================================
 
void ForceField::imposeWorkerForce()
{
		map<int, ScalarPoint>::iterator it;
		
		for (it=workerPos.begin(); it != workerPos.end(); it++) {
			// cout << "Pos: (" << it->second.x << "," << it->second.y << ")" << endl;
			for (int y=(it->second.y)-workerOffset; y <= (it->second.y)+workerOffset; ++y) {
				for (int x=(it->second.x)-workerOffset; x <= (it->second.x)+workerOffset; ++x) {
						if (y>=0 && x>=0 && x < width && y < height) {
							if (y==it->second.y && x==it->second.x) {
								dynamicFieldOutbound [y*width+x] = dynamicFieldOutbound[y*width+x] - workerForce; 
								//fieldOutbound [y*width+x] = fieldOutbound[y*width+x] -workerForce;
								//cout <<workerForce << " ";
							}
							else
							{
								dynamicFieldOutbound [y*width+x] = dynamicFieldOutbound[y*width+x] - 
												1/sqrt( (x-workerPos[it->first].x) *
																(x-workerPos[it->first].x) +
															  (y-workerPos[it->first].y) *
															  (y-workerPos[it->first].y)
															)	*	WORKER_FORCE;
								//fieldOutbound [y*width+x] = fieldOutbound[y*width+x] - 
								//				1/sqrt( (x-workerPos[it->first].x) *
								//								(x-workerPos[it->first].x) +
								//							  (y-workerPos[it->first].y) *
								//							  (y-workerPos[it->first].y)
								//							)	*	WORKER_FORCE;
							}
						}
				}
				//cout << endl;
			}
		}
}

//=================================================================
 
void ForceField::imposeSheepForce()
{
		map<int, ScalarPoint>::iterator it;
		
		for (it=sheepPos.begin(); it != sheepPos.end(); ++it) {
			for (int y=it->second.y-sheepOffset; y <= it->second.y+sheepOffset; ++y) {
				for (int x=it->second.x-sheepOffset; x <= it->second.x+sheepOffset; ++x) {
						if (y>=0 && x>=0) {
							if (y==it->second.y && x==it->second.x) {
								dynamicFieldOutbound [y*width+x] = dynamicFieldOutbound[y*width+x] -sheepForce; 
								//dynamicFieldOutbound [y*width+x] = dynamicFieldOutbound[y*width+x] -sheepForce;
							}
							else
							{
								dynamicFieldOutbound [y*width+x] = dynamicFieldOutbound[y*width+x] - 
												1/sqrt( (x-sheepPos[it->first].x) *
																(x-sheepPos[it->first].x) +
															  (y-sheepPos[it->first].y) *
															  (y-sheepPos[it->first].y)
															)	*	SHEEP_FORCE;
								//fieldOutbound [y*width+x] = fieldOutbound[y*width+x] - 
								//				1/sqrt( (x-sheepPos[it->first].x) *
								//								(x-sheepPos[it->first].x) +
								//							  (y-sheepPos[it->first].y) *
								//							  (y-sheepPos[it->first].y)
								//							)	*	SHEEP_FORCE;
							}
							
						}
				}
			}
		}
}
//======================================================================

void ForceField::imposeBoundaryForce(GameObj *gob)
{
		int x1 = *gob->sod.x1;
		int y1 = *gob->sod.y1;
		int x2 = *gob->sod.x2;
		int y2 = *gob->sod.y2;
		
		cout << "boundaryForce = " << boundaryForce <<endl;
		
			for (int y=y1-boundaryOffset; y <= y1+boundaryOffset; ++y) {
				for (int x=x1-boundaryOffset; x <= x1+boundaryOffset; ++x) {
					if (y>=0 && x>=0  && y < height && x < width) {
						if (y==y1 && x==x1) {
							staticFieldOutbound [y*width+x] = staticFieldOutbound[y*width+x] -boundaryForce; 
							//fieldOutbound [y*width+x] = fieldOutbound[y*width+x] -boundaryForce;
						}
						else
						{
							staticFieldOutbound [y*width+x] = staticFieldOutbound[y*width+x] - 
											1/sqrt( (x-x1)*(x-x1) + (y-y1)*(y-y1))	
												*	BOUNDARY_FORCE;
							//fieldOutbound [y*width+x] = fieldOutbound[y*width+x] - 
							//				1/sqrt( (x-x1)*(x-x1) + (y-y1)*(y-y1))	
							//					*	BOUNDARY_FORCE;
						}
					}
				}
			}	
			for (int y=y2-boundaryOffset; y <= y2+boundaryOffset; ++y) {
				for (int x=x2-boundaryOffset; x <= x2+boundaryOffset; ++x) {
					if (y>=0 && x>=0 && y < height && x < width) {
						if (y==y2 && x==x2) {
							staticFieldOutbound [y*width+x] = staticFieldOutbound[y*width+x] -boundaryForce; 
							//fieldOutbound [y*width+x] = fieldOutbound[y*width+x] -boundaryForce;
						}
						else
						{
							staticFieldOutbound [y*width+x] = staticFieldOutbound[y*width+x] - 
											1/sqrt( (x-x2)*(x-x2) + (y-y2)*(y-y2))	
												*	BOUNDARY_FORCE;
							//fieldOutbound [y*width+x] = fieldOutbound[y*width+x] - 
							//				1/sqrt( (x-x2)*(x-x2) + (y-y2)*(y-y2))	
							//					*	BOUNDARY_FORCE;
						}
					}
				}
				
			}
}
//========================================================================
void ForceField::imposeMineralForceFieldNC() 
{
	std::vector<GameObj*>::iterator mineralIt;
	for (int y=0; y < height; y++) {
		for (int x=0; x < width; x++) {
			for (mineralIt=minerals.begin();mineralIt!=minerals.end();++mineralIt) {
				if (*(*mineralIt)->sod.x == x && *(*mineralIt)->sod.y == y) {
					staticFieldOutbound[y*width+x] = staticFieldOutbound[y*width+x] + mineralForce;
					//fieldOutbound[y*width+x] = fieldOutbound[y*width+x] + mineralForce; 
				}
				else {
					staticFieldOutbound[y*width+x] = staticFieldOutbound[y*width+x] + 
						1/sqrt( (*(*mineralIt)->sod.x - x) * (*(*mineralIt)->sod.x - x)  
									+ (*(*mineralIt)->sod.y - y) * (*(*mineralIt)->sod.y - y))
							* mineralForce;
					//fieldOutbound[y*width+x] = fieldOutbound[y*width+x] + 
					//	1/sqrt( (*(*mineralIt)->sod.x - x) * (*(*mineralIt)->sod.x - x)  
					//				+ (*(*mineralIt)->sod.y - y) * (*(*mineralIt)->sod.y - y))  
					//		* mineralForce; 
				}
			}
		}
	} 	
}
//=================================================================
 
void ForceField::compute_actions()
{

	// cout << "compute_actions" << endl;
	const Game &game = gsm.get_game();
  const sint4 cid = game.get_client_player();
	const Game::ObjCont &objs = game.get_objs(cid);
	
	FORALL (objs, it) {
	
		GameObj *gob = (*it)->get_GameObj();
		
		if (gob && gob->sod.in_game && *gob->sod.x >= 0 && !gob->is_dead()) {
			if (gob->bp_name() == "worker" ) { //&& gob->get_obj_id(gob) == myWorker
				
				int reach = *gob->sod.max_speed;
				int speed = *gob->sod.max_speed; 
				int dist;
				sint4 target_x = *gob->sod.x;
				sint4 target_y = *gob->sod.y;
				double max = -32000.0;
				double current = 0.0;
				
				debug << "current position: ( " << *gob->sod.x << " , " << *gob->sod.y << " )" ;
				debug << "force: " << dynamicFieldOutbound[*gob->sod.y * width + *gob->sod.x] << endl;
				
				for (int y = *gob->sod.y - reach; y <= *gob->sod.y + reach; ++y) {
					for (int x= *gob->sod.x - reach; x <= *gob->sod.x + reach; ++x) {
						// if (outbound)
						
							current =  dynamicFieldOutbound[y*width+x] 
											 - dynamicFieldOutbound[*gob->sod.y * width + *gob->sod.x];
							dist = sqrt(x*x + y*y);
							debug << "difference to pos ( " << x << " , " << y << " ) = " << current << endl;
							if (current > max) { // && dist <= reach) {
								max = current;
								target_x = x;
								target_y = y;
								speed = ceil(sqrt((*gob->sod.y-y)*(*gob->sod.y-y)+
														 			(*gob->sod.x-x)*(*gob->sod.x-x)));
							}
						// else
						/*
							max = fieldOutbound[y*width+x] - 
									fieldOutbound[*gob->sod.y * width + *gob->sod.x]
						*/
					}
				}
				
				Vector<sint4> params;
				params.push_back(target_x);
				params.push_back(target_y);
				params.push_back(speed);
				
				debug << "goto ( " << target_x << " , " << target_y << " ) ; speed " << speed;
				debug <<  "force: " << dynamicFieldOutbound[target_y*width+target_x] << endl; 
				debug << " --------------------------------------------------------------" << endl;
 /*       
        cout << gob->bp_name() <<  gob->get_obj_id(gob) << " von (" << *gob->sod.x;
        cout << ","<< *gob->sod.y <<") nach (" << target_x << "," << target_y << ")"; 
        cout << " max: " << max << " current: " << current << endl; 
*/
				
				// if (gob->get_obj_id(gob) == myWorker)
					gob->set_action("move", params);

			}
		}
	
	}
}
//======================================================================
void ForceField::outputResult()
{
	vector<GameObj*>::iterator workerIt;
	vector<GameObj*>::iterator mineralIt;
	bool nearMineral;
	int workersAtMineral = 0;
	
	outFile << ccForce << " ";
	outFile << mineralForce << " ";
	outFile << boundaryForce << " ";
	outFile << boundaryOffset << " ";
	outFile << workerForce << " ";
	outFile << workerOffset << " ";
	outFile <<  sheepForce << " ";	
	outFile << sheepOffset << " ";
	outFile << noMineralClustering << " ";
/*	
	outFile << "ccForce " << ccForce << " ";
	outFile << "mineralForce " << mineralForce << " ";
	outFile << "boundaryForce " << boundaryForce << " ";
	outFile << "boundaryOffset " << boundaryOffset << " ";
	outFile << "workerForce " << workerForce << " ";
	outFile << "workerOffset " << workerOffset << " ";
	outFile << "sheepForce " <<  sheepForce << " ";	
	outFile << "sheepOffset " << sheepOffset << " ";
	outFile << "noMineralClustering " << noMineralClustering << " ";
	// outFile << "interClusterDistance " << interClusterDistance << endl;
	*/
	for (workerIt = workers.begin(); workerIt != workers.end(); ++workerIt) {
		for (mineralIt = minerals.begin(); mineralIt != minerals.end(); ++mineralIt) {
			nearMineral = (sint4)(Game::distance(**workerIt,**mineralIt)) <= WORKER_MINERAL_DISTANCE_THRESHOLD;
			if (nearMineral) {
				++workersAtMineral;
				break;
			}
		}
	}

	outFile << workersAtMineral << endl;
}

//======================================================================
bool ForceField::handle_event(const Event &e)
{
	// cout << "handle event" << endl;
	

  if (e.get_who() == GameStateModule::FROM) {

    if (e.get_what() == GameStateModule::VIEW_MSG) {

      Game &game = gsm.get_game();
      sint4 vf = game.get_view_frame();
            
      if (vf == 512) {
      	ForceField::outputResult();
      	exit(0);
      }
      
      for (int i=0;i<num;i++) 
      	dynamicFieldOutbound[i] = staticFieldOutbound[i]; 
      
      //if (!gsm.get_changes().new_objs.empty())
      //		printField(dynamicFieldInbound);
      
      // new objectsdynamic
      FORALL (gsm.get_changes().new_objs, i) {
        GameObj *gob = (*i)->get_GameObj();
        if (gob == 0) continue;
//        if (*gob->sod.owner != me) continue;


				// set force field of the control-center
				if (gob->bp_name() == "controlCenter") {
					cout << "control Center found at (" << *gob->sod.x << "," <<  *gob->sod.y << ") " << endl;
					cout << "von  (" << *gob->sod.x1 << "," <<  *gob->sod.y1 << ") " << endl;
					cout << "bis (" << *gob->sod.x2 << "," <<  *gob->sod.y2 << ") " << endl; 
					for (int y=*gob->sod.y-ccOffset; y <= *gob->sod.y+ccOffset; ++y) {
						for (int x=*gob->sod.x-ccOffset; x <= *gob->sod.x+ccOffset; ++x) {
							if (x != *gob->sod.x || y != *gob->sod.y)
								staticFieldOutbound[y*width+x] = staticFieldOutbound[y*width+x] -
													1/sqrt((x-*gob->sod.x)*(x-*gob->sod.x)+
																 (y-*gob->sod.y)*(y-*gob->sod.y))
														*	ccForce;
							else 
								staticFieldOutbound[y*width+x] = staticFieldOutbound[y*width+x] - ccForce;
							//fieldOutbound[y*width+x] = -1 * fieldInbound[y*width+x];
						}
					}
					//printField(staticFieldInbound, ScalarPoint(*gob->sod.x,*gob->sod.y), 32);
					cout << "ready with CC force field" << endl;
					
				}		
				
				
				// for all minerals, find loation and group them into clusters
				// clusters are determined by distance: 
				// 		two minerals close enough are grouped into one cluster
				//		this works recursively	 
				if (gob->bp_name() == "mineral") {
					
					//cout << gob->bp_name() << " ," << gob->get_obj_id(gob) << " (" << *gob->sod.x;
        	//cout << ","<< *gob->sod.y <<")" << endl;
        	minerals.push_back(gob);
        	
        	if (noMineralClustering) {
        		
        	}
        		
       		else { //mineral clustering
						
						multimap<int, GameObj*>::iterator it;
						std::vector<int> withinCluster; // vector contains all cluster numbers the 
																						// current mineral needs to be assigned to.
						std::vector<int>::iterator vIt; //  corresponding iterator	
						
																			
						if (numMineralClusters == 0) {  // first mineral is assigned to cluster 1
							++numMineralClusters; 
							mineralClusters.insert(pair<int, GameObj*>(numMineralClusters,gob));
						}
						else { // all other minerals will be ckecked against all already encountered minerals.
							for (it = mineralClusters.begin(); 
									 it != mineralClusters.end(); ++it) {
								GameObj *gobC = it->second;
								
								if (gobC == gob) continue; // do not make all the stuff for a self comparison!
								
								// cout << gobC->bp_name() << gobC->get_obj_id(gobC);
								// cout << "( " << *gobC->sod.x << "," << *gobC->sod.y << " )";
								
								// calculate distance from current GameObj to all 
								// GameObj already assigned to clusters.
								double dist = sqrt((*gob->sod.x - *gobC->sod.x)*
																	 (*gob->sod.x - *gobC->sod.x)
																	+(*gob->sod.y - *gobC->sod.y)*
																	 (*gob->sod.y - *gobC->sod.y)
																	);	
																	
								//cout << "(" << *gob->sod.x << "," << *gob->sod.y << ") " << dist << " ";
								//cout << "( " << *gobC->sod.x << "," << *gobC->sod.y << " )" << endl;
								
								if ( dist <= interClusterDistance) { 
																								// the current mineral(gob) needs to be assigend to
																								// the same cluster as the one it is compared to (gobC)
																						
									if (withinCluster.size() > 0) { //mineral already assigned to a cluster 
									 //cout << "assigned to: " << withinCluster.size() << endl;
									
										vIt = find(withinCluster.begin(), withinCluster.end(), it->first);
										if (vIt == withinCluster.end()) { // the mineral (gob) is not yet assigned to the
																											// cluster of the compare mineral
											withinCluster.push_back(it->first); // add the no of the new cluster to the
																													// within cluster vector
											//cout << "already assigned, but not to cluster " << it->first;
											//cout << " add it to withinCluster vector" << endl;
										}	
/*										else {
											//cout << "already assigned to same cluster: " << it->first << endl;
										} */
									}
									else{ // mineral is no yet assigned to a cluster
										// Add the mineral to the cluster of the compare mineral	
										mineralClusters.insert(pair<int, GameObj*>(it->first,gob));
										withinCluster.push_back(it->first);
										// cout << "not yet assigned, assign it to: " << it->first << endl;
									}
								}
								
							}// end for
							
							if (withinCluster.size() == 0 ) { // mineral has not been assigned to an already
																								// existing cluster
									++numMineralClusters; 
									mineralClusters.insert(pair<int, GameObj*>(numMineralClusters,gob)); 
									withinCluster.push_back(numMineralClusters);
							}
/*
							if (withinCluster.size() == 1) {
								// do nothing. Mineral has been assigned to one existing cluster.
								// i.e.: no new cluster needs to be created
								//			 no clusters need to be joined 
							}
*/
							if (withinCluster.size() > 1) { // now we need to join all the clusters the mineral
																						// belongs to.
							
								int finalNo = -1;
							
								// cout << "join into ";
							
								for (vIt= withinCluster.begin(); vIt != withinCluster.end(); ) {
									if (finalNo==-1) {
										finalNo = *vIt;
										// cout << finalNo << ": ";
									}
									else {
										// cout << *vIt << " ";
										for (it = mineralClusters.begin(); it != mineralClusters.end();) {
											if (it->first == *vIt) {
												mineralClusters.insert(pair<int, GameObj*>(finalNo, it->second));
												mineralClusters.erase(it); 
											}
											++it;
										}
									}
									++vIt;
								}
								cout << endl;
							}
						}// end else
					}// end minerral clustering	
					// cout << "mineral" << endl;
				}// end of mineral block	

				
				if(gob->bp_name() == "worker") {
					if (!myWorker) myWorker = gob->get_obj_id(gob);
					workerPos.insert(pair<int,ScalarPoint> 
														(gob->get_obj_id(gob), 
														 ScalarPoint(*gob->sod.x,*gob->sod.y)
														));
					workers.push_back(gob);
					
					
				} //end of worker block 

/*
				if(gob->bp_name() == "sheep") {
					// cout << "sheep" << endl;
					sheepPos.insert(pair<int,ScalarPoint> 
														(gob->get_obj_id(gob), 
														 ScalarPoint(*gob->sod.x,*gob->sod.y)
														));
				} //end of sheep block 				
*/				
      }// end of FORALL new_objects
			
			// we have looped over all new minerals, there are no more 
			// now, we are adding the impact of the minerals to the force field
			// this is only done once, hence the boolean ffMinerals (=true as soon as the field is set)
			// 2 steps:
			// 1: create cluster centers and store their positions 
			// 		use map to assign own keys, keys need to be consistent for position and no of minerals
			//		also store the number of minerals assigned to each cluster
			// 2: the cluster centers are the origin of the force
			//    loop over force field
			//		for every point of the force field loop over the cluster centers
			//		calculate impact and subtract from existing value
			//		impact: base impact per mineral, multiply by number, 
			// 						decay same as for control center (1/r for the moment)
			
			if (!ffMinerals) {
			
				if (noMineralClustering) {
					imposeMineralForceFieldNC();
				}
				
				else {
					multimap<int, GameObj*>::iterator clusterIt;
					std::map <int, ScalarPoint> clusterCenters;
					std::map <int, int> clusterY;
					std::map<int, int> elemCount;
					std::map<int, ScalarPoint>::iterator centerIt;
					// map<int,int>::iterator elemIt;
				
					// Step 1:
					for (clusterIt = mineralClusters.begin(); clusterIt != mineralClusters.end(); ++clusterIt) {
						centerIt = clusterCenters.find(clusterIt->first);
						if (centerIt == clusterCenters.end()) {	// no clusterCenter yet
							clusterCenters.insert(pair<int, ScalarPoint>
												(clusterIt->first,
												ScalarPoint(*(clusterIt->second)->sod.x,*(clusterIt->second)->sod.y)));
							elemCount.insert(pair<int,int> (clusterIt->first,1));
						
/*							// debug print-out					
							cout << clusterIt->first << ","<< *(clusterIt->second)->sod.x;
							cout << "," <<*(clusterIt->second)->sod.y << endl;  */
						}
						else {
							elemCount[clusterIt->first] += 1;
							// the following calculation rounds doubles to integers in each step:
							// this will result in summing up rounding errors !
							clusterCenters[clusterIt->first] = 
								ScalarPoint(
															clusterCenters[clusterIt->first].x + 
															(*(clusterIt->second)->sod.x - clusterCenters[clusterIt->first].x) /
																elemCount[clusterIt->first]
													  ,
													  	clusterCenters[clusterIt->first].y + 
															(*(clusterIt->second)->sod.y - clusterCenters[clusterIt->first].y) /
																elemCount[clusterIt->first]
														);
														
						}
					}
				
				
/*				
					//print the clusters and the elemCount for each cluster
					// debugging only
					for (centerIt = clusterCenters.begin();centerIt != clusterCenters.end();++centerIt) {
						cout << "cluster: " << centerIt->first;
						cout << " (" << clusterCenters[centerIt->first].x;
						cout << "," << clusterCenters[centerIt->first].y;
						cout << ") number of minerals: " << elemCount[centerIt->first];
						cout << endl;
					
						for (clusterIt = mineralClusters.begin(); clusterIt != mineralClusters.end(); ++clusterIt) {
							if (clusterIt-> first == centerIt-> first) {
								cout << "\t (" << *(clusterIt->second)->sod.x << ",";
								cout << *(clusterIt->second)->sod.y << ")" << endl;
							}
						} 
				
					}
*/				
				 
					//Step 2:
					for (int y=0; y < height; ++y) {
						for (int x=0; x < width; ++x) {
							for (centerIt = clusterCenters.begin();centerIt != clusterCenters.end();++centerIt) {
								
								if (x != clusterCenters[centerIt->first].x || y != clusterCenters[centerIt->first].y) {
									 staticFieldOutbound [y*width+x] = staticFieldOutbound[y*width+x] + 
													1/sqrt( (x-clusterCenters[centerIt->first].x) *
																	(x-clusterCenters[centerIt->first].x) +
																  (y-clusterCenters[centerIt->first].y) *
																  (y-clusterCenters[centerIt->first].y)
																)	*	MINERAL_FORCE * elemCount[centerIt->first];
								}
								else  {
									staticFieldOutbound[y*width+x] = staticFieldOutbound[y*width+x] + 
																						mineralForce * elemCount[centerIt->first];
									//cout << "setting mineral force field";
								}
							}
							//fieldOutbound[y*width+x] = -1 * fieldInbound[y*width+x];
						}
					}
				}// end mineral clustering
	
				cout << "ready with Mineral force field" << endl;
				//printField(fieldOutbound, ScalarPoint(216, 408), 16);
				
				
								ffMinerals = true;				
				//multimap<int, GameObj*>::iterator it;	
				//for (it = mineralClusters.begin(); it != mineralClusters.end(); ++it) {
				//	cout << it->second->bp_name() << it->second->get_obj_id(it->second);
				//	cout << ": [ " << it->first << " (" << *(it->second)->sod.x << ",";
				//	cout << *(it->second)->sod.y << ") ]" << endl;
				//}	
			}
			
			FORALL (gsm.get_changes().new_boundaries, i) {
				GameObj *gob = (*i)->get_GameObj();
        if (gob == 0) continue;
        
        imposeBoundaryForce(gob);
      }
      
      if (!ffBoundary) {
      	printFieldGnuplot(staticFieldOutbound);
 	    	ffBoundary=true;
      }
			
	 		FORALL (gsm.get_changes().changed_objs, i) {
	 		
				GameObj *gob = (*i)->get_GameObj();
        if (gob == 0) continue;
        //if (*gob->sod.owner != me) continue;
        
        if(gob->bp_name() == "worker") {
        	workerPos[gob->get_obj_id(gob)] = ScalarPoint(*gob->sod.x,*gob->sod.y);
        	if (gob->get_obj_id(gob) == myWorker) {
						myWorkerData << "start_f_pos: " << gob->smd.start_f_pos.x << ", " << gob->smd.start_f_pos.y << endl;
						myWorkerData << "target_f_pos: " << gob->smd.target_f_pos.x << ", " << gob->smd.target_f_pos.y << endl;
						myWorkerData << "f_delta: " << gob->smd.f_delta.x << ", " << gob->smd.f_delta.y << endl;
						myWorkerData << "fine_pos: " << gob->smd.fine_pos.x << ", " << gob->smd.fine_pos.y << endl;
						myWorkerData << "fine_heading: " << gob->smd.fine_heading.x << ", " << gob->smd.fine_heading.y << endl;
						myWorkerData << "============================================================================" << endl;
					}
        }
    
        if(gob->bp_name() == "sheep") {
        	sheepPos[gob->get_obj_id(gob)] = ScalarPoint(*gob->sod.x,*gob->sod.y);
        }
		
	 		}
      
      // dead objects
      FORALL (gsm.get_changes().dead_objs, i) {
        GameObj *gob = (*i)->get_GameObj();
        if (gob == 0) continue;
	
      }
			
			imposeSheepForce();
			imposeWorkerForce();
      
      
      compute_actions();
      gsm.send_actions();
      
      return true;
    }
  }

  return false;
}


/* =============================================================================
			***   old boundary solution    ****
			
			FORALL (gsm.get_changes().new_boundaries, i) {
				GameObj *gob = (*i)->get_GameObj();
        if (gob == 0) continue;
        cout << gob->bp_name() << gob->get_obj_id(gob);
				cout << "( " << *gob->sod.x1 << "," << *gob->sod.y1 << " ) - (";
				cout << *gob->sod.x2 << "," << *gob->sod.y2 << " )" << endl;
        // for all boundaries, find loation and group them into clusters
				// clusters are determined by connectivity: 
				// 		two connected boundaries (i.e. either starting point or end point 
				//    equals starting or end point of other boundary) are grouped into one cluster
				//		this works recursively	 
				
				multimap<int, GameObj*>::iterator it;
				std::vector<int> withinCluster; // vector contains all cluster numbers the 
																				// current boundary needs to be assigned to.
				std::vector<int>::iterator vIt; //  corresponding iterator														
				if (numBoundaryClusters == 0) {  // first boundary is assigned to cluster 1
					++numBoundaryClusters; 
					boundaryClusters.insert(pair<int, GameObj*>(numBoundaryClusters,gob));
				}
				else { // all other boundaries will be ckecked against all already encountered boundaries.
					for (it = boundaryClusters.begin(); it != boundaryClusters.end(); ++it) {
						GameObj *gobC = it->second;
						if (gobC == gob) continue;
						//cout << gobC->bp_name() << gobC->get_obj_id(gobC);
						//cout << "( " << *gobC->sod.x1 << "," << *gobC->sod.y1 << " )";
						
						// check if current GameObj is connected any of the  
						// GameObj already assigned to clusters.
						bool connected = (*gob->sod.x1 == *gobC->sod.x1 && *gob->sod.y1 == *gobC->sod.y1) || 
														 (*gob->sod.x1 == *gobC->sod.x2 && *gob->sod.y1 == *gobC->sod.y2) ||
														 (*gob->sod.x2 == *gobC->sod.x2 && *gob->sod.y2 == *gobC->sod.y2) ||
														 (*gob->sod.x2 == *gobC->sod.x1 && *gob->sod.y2 == *gobC->sod.y1) ;

																
						if ( connected ) { // the current boundary(gob) needs to be assigend to
																							// the same cluster as the one it is compared to (gobC)
							if (withinCluster.size() > 0) { //boundary already assigned to a cluster 
								
								vIt = find(withinCluster.begin(), withinCluster.end(), it->first);
								if (vIt == withinCluster.end()) { // the boundary (gob) is not yet assigned to the
																									// cluster of the compare boundary
									withinCluster.push_back(it->first); // add the no of the new cluster to the
																												// within cluster vector
								}	
							}
							else{ // boundary is no yet assigned to a cluster
									// Add the boundary to the cluster of the compare boundary	
									boundaryClusters.insert(pair<int, GameObj*>(it->first,gob));
									withinCluster.push_back(it->first);
							}
						}
						
					}// end for
						
					if (withinCluster.size() == 0 ) { // boundary has not been assigned to an already
																							// existing cluster
								++numBoundaryClusters; 
								boundaryClusters.insert(pair<int, GameObj*>(numBoundaryClusters,gob)); 
								withinCluster.push_back(numBoundaryClusters);
						}
						
						if (withinCluster.size() > 1) { // now we need to join all the clusters the boundary
																						// belongs to.
							
							int finalNo = -1;
							
							
							for (vIt= withinCluster.begin(); vIt != withinCluster.end(); ) {
								if (finalNo==-1) finalNo = *vIt;
								else {
									for (it = boundaryClusters.begin(); it != boundaryClusters.end(); ) {
										if (it->first == *vIt) {
											boundaryClusters.insert(pair<int, GameObj*>(finalNo, it->second));
											boundaryClusters.erase(it); 
										}
										++it;
									}
								}
								++vIt;
							}
					}
				}// end else (ckeck of boundary against all other boundaries)		
			}// end FORALL new_boundaries	
			
			
			// we have looped over all new boundaries, there are no more 
			// now, we are adding the impact of the boundaries to the force field
			// this is only done once, hence the boolean ffBoundary (=true as soon as the field is set)
			// 2 steps:
			// 1: create cluster centers and store their positions 
			// 		use map to assign own keys, keys need to be consistent for position and no of boundaries
			//		also store the number of boundary-points assigned to each cluster
			// 2: the cluster centers are the origin of the force
			//    loop over force field
			//		for every point of the force field loop over the cluster centers
			//		calculate impact and subtract from existing value
			//		impact: base impact per boundary, multiply by number, 
			// 						decay same as for control center (1/r for the moment)
			
			if (!ffBoundary) {
				multimap<int, GameObj*>::iterator clusterIt;
				std::map <int, ScalarPoint> clusterCenters;
				std::map<int, int> elemCount;
				std::map<int, ScalarPoint>::iterator centerIt;
				// map<int,int>::iterator elemIt;
				cout << "ffBoundary = " << ffBoundary << endl;
				// Step 1:
				for (clusterIt = boundaryClusters.begin();clusterIt != boundaryClusters.end();++clusterIt) {
					centerIt = clusterCenters.find(clusterIt->first);
					if (centerIt == clusterCenters.end()) {	// no clusterCenter yet
						clusterCenters.insert(pair<int, ScalarPoint>
										(clusterIt->first,
										 ScalarPoint((*(clusterIt->second)->sod.x1 + *(clusterIt->second)->sod.x2) / 2,
											  				 (*(clusterIt->second)->sod.y1 + *(clusterIt->second)->sod.y2) / 2
															  )
										));
						elemCount.insert(pair<int,int> (clusterIt->first,1));
					}
					else {
						elemCount[clusterIt->first] += 1;
						clusterCenters[clusterIt->first] = 
							ScalarPoint(
														clusterCenters[clusterIt->first].x + 
														(((*(clusterIt->second)->sod.x1 + *(clusterIt->second)->sod.x2) / 2)
																- clusterCenters[clusterIt->first].x)
																	/ elemCount[clusterIt->first]
												  ,
												  	clusterCenters[clusterIt->first].y + 
														(((*(clusterIt->second)->sod.y1 + *(clusterIt->second)->sod.y2) / 2) 
																	- clusterCenters[clusterIt->first].y)
																		/ elemCount[clusterIt->first]
													);
					}
				}
				cout << "trying to print now" << endl;
				//print the clusters and the elemCount for each cluster
				// debugging only
				for (centerIt = clusterCenters.begin();centerIt != clusterCenters.end();++centerIt) {
					cout << "cluster: " << centerIt->first;
					cout << " (" << clusterCenters[centerIt->first].x;
					cout << "," << clusterCenters[centerIt->first].y;
					cout << ") number of boundaries: " << elemCount[centerIt->first];
					cout << endl;
					
					for (clusterIt = boundaryClusters.begin(); clusterIt != boundaryClusters.end(); ++clusterIt) {
						if (clusterIt-> first == centerIt-> first) {
							cout << "\t (" << *(clusterIt->second)->sod.x1 << ",";
							cout << *(clusterIt->second)->sod.y1 << ") - (" << *(clusterIt->second)->sod.x2 << ",";
							cout << *(clusterIt->second)->sod.y2 << ") - (" << endl;
						}
					} 
				
				}
				 
				//Step 2:
				for (int y=0; y < height; ++y) {
					for (int x=0; x < width; ++x) {
						for (centerIt = clusterCenters.begin();centerIt != clusterCenters.end();++centerIt) {
								
							if (x != clusterCenters[centerIt->first].x || y != clusterCenters[centerIt->first].y) {
								 fieldInbound [y*width+x] = fieldInbound[y*width+x] - 
												1/sqrt( (x-clusterCenters[centerIt->first].x) *
																(x-clusterCenters[centerIt->first].x) +
															  (y-clusterCenters[centerIt->first].y) *
															  (y-clusterCenters[centerIt->first].x)
															)	*	BOUNDARY_FORCE; // * elemCount[centerIt->first];
								fieldOutbound [y*width+x] = fieldOutbound[y*width+x] - 
												1/sqrt( (x-clusterCenters[centerIt->first].x) *
																(x-clusterCenters[centerIt->first].x) +
															  (y-clusterCenters[centerIt->first].y) *
															  (y-clusterCenters[centerIt->first].x)
															)	*	BOUNDARY_FORCE; // * elemCount[centerIt->first];
							}
							else 
								fieldInbound[y*width+x] = fieldInbound[y*width+x] - 
																					boundaryForce; // * elemCount[centerIt->first];
								fieldOutbound[y*width+x] = fieldOutbound[y*width+x] - 
																					boundaryForce; // * elemCount[centerIt->first];
						}
					}
				}
			
				ffBoundary = true;
				cout << "ready with Boundary force field" << endl;
				//multimap<int, GameObj*>::iterator it;	
				//for (it = mineralClusters.begin(); it != mineralClusters.end(); ++it) {
				//	cout << it->second->bp_name() << it->second->get_obj_id(it->second);
				//	cout << ": [ " << it->first << " (" << *(it->second)->sod.x << ",";
				//	cout << *(it->second)->sod.y << ") ]" << endl;
				//}	
			}
						***   end old boundary solution   ***
================================================================================
*/
