// $Id: myAi_main.C,v 1.1.1.1 2007/04/09 23:50:26 pjungkun Exp $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

// this is the main file of the sample ORTS AI client
// options are specified by the opt.put lines below
// to test it start the ORTS server (bin/orts) and then run bin/sampleai [options]

// SampleEventHandler.C contains the code for accessing the game state

#include "GfxGlobal.H"
#include "Options.H"
#include "GfxModule.H"
#include "GameStateModule.H"
#include "ShutDown.H"
#include "SDLinit.H"
#include "Game.H"

#include "SampleEventHandler.H"
#include "Predict.H"
#include "ForceField.H"

static char *VERSION = "sampleai v0.1";

using namespace std;

void add_options()
{
  Options opt("SampleClient");
  opt.put("-usegfx", "display world in 3d");
  opt.put("-cinfo", "display game state changes");
  opt.put("-tinfo", "display tile info");
  opt.put("-oinfo", "display object info");
  opt.put("-minfo", "display map info");  
  opt.put("-ccforce", (int)-1, "control center force");
  opt.put("-mforce", (int)-1, "mineral force");
  opt.put("-bforce", (int)-1, "boundary force");
  opt.put("-wforce", (int)-1, "worker force");
  opt.put("-sforce", (int)-1, "sheep force");
  opt.put("-cdist", (int)-1, "max dist for clustering minerals together");
  opt.put("-ccrange", (int) -1, "impact range of control center force");
  opt.put("-wrange", (int)-1, "impact range of worker force");
  opt.put("-brange", (int)-1, "impact range of boundary force");
  opt.put("-srange", (int)-1, "impact range of sheep force");
  opt.put("-nomc", "no mineral clustering"); 
  opt.put("-outfile", string(""), "file to which result is written");
}

// loop is called

class Looper : public LoopFunctor
{
public :

  Looper(SampleAIState *state_) : state(state_) {}
  
  void loop() const {

    if (state->gfxm) {

      if (state->gfxm->get_quit()) { state->quit = true; exit(0); } // should return

      bool refresh;
      Options::get("-refresh", refresh);
      
      if (refresh) {

	// maximal gfx refresh rate
	
	if (!state->just_drew) 
	  state->gfxm->draw();
	else
	  state->just_drew = false;
      }
    }
  
    // looks for server messages
    // if one or more arrived, calls handlers
    // if not, sleeps for one millisecond
    if (!state->gsm->recv_view()) SDL_Delay(1);
  }

private:
  SampleAIState *state;
};


int main(int argc, char **argv)
{
  
  signals_shut_down(true);  
  
  // populate opt with options from various modules

  add_options();
  GameStateModule::Options::add();
  GfxModule::Options::add();
  Game::Options::add();

  // process command line arguments
  if (Options::process(argc, argv, cerr, VERSION)) exit(20);

  // initialize SDL
  SDLinit::video_init();
  SDLinit::network_init();
  
  // populate option objects with command line values
  // and create modules
  
  GameStateModule::Options gsmo;
  GameStateModule gsm(gsmo);
 
  GfxModule::Options gfxmo;
  GfxModule *gfxm = 0;
  
  // testing for boundary Force argument
  
  //int boundaryForce;
  //Options::get("-bforce", boundaryForce);
  //cout << "boundaryForce = " << boundaryForce << endl;

  // change seed value to time(0) if 0

  sint4 seed;
  Options::get("-seed", seed);

  if (!seed) seed = time(0);

  Options::set("-seed", seed);
  cout << "seed=" << seed << endl;

  bool use_gfx;
  Options::get("-usegfx", use_gfx);

  if (use_gfx) gfxm = new GfxModule;

  // gives event handler access to the state
  SampleAIState state(&gsm, gfxm);

  SampleEventHandler sev(state, seed);

  // gsm events are handled in sev
   gsm.add_handler(&sev);
   
    // connect game state module to server
  if (!gsm.connect()) ERR("connection problems");
  
  //====================================  --PJ
  //Predict predict(gsm);
  //gsm.add_handler(&predict);
  ForceField forceField(gsm);
  gsm.add_handler(&forceField);
  //===================================== --PJ

  cout << "added handlers" << endl;
 

  Looper l(&state);

  if (use_gfx) {

    // gfx mode

    glutInit(&argc, argv);
    gfxm->init(gsm.get_game(), gsm.get_changes(), gsm.get_action_changes(), gfxmo);

    // game loop (could become a thread)
    gfxm->start_loop(&l);

  } else {

    // no gfx
    
    while (!state.quit) l.loop();
  }
  
  delete gfxm;
  return 0;
}
