// // $Id: SampleEventHandler.C,v 1.1 2007/04/25 20:40:29 shstoyko Exp $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

// A sample client for ORTS that shows how to access client data

// todo/questions:
//
// game scripts:
// - what to do with workers during build time
// - similarly, how to define mining?
// - how is attack range defined (distance between centers/hulls)?
//
// client:
// - how to access global player data?
// - how to compute the set of visible tiles?
// - how to compute tiles visible from a unit?
// - how to decide what can be attacked by a specific unit?

// FORS/FORALL/ERR macros are defined in orts3/libs/kernel/src/Global.H

#include "SampleEventHandler.H"
#include "GameStateModule.H"
#include "GfxModule.H"
#include "Game.H"
#include "GameChanges.H"
#include "GameTile.H"
#include "ServerObjData.H"
#include "Options.H"
#include "GUI.H"

#include "Distance.H"

#include "LocalWorker.h"
#include "WorkerController.h"
//#include "DrawOnTerrain2D.H"


using namespace std;

// tile types from core.bp
// fixme: game script reflection would help here
// in the competition we won't use water tiles
// TILE_CLIFF: non-traversable by ground units

enum { TILE_UNKNOWN=0, TILE_WATER=1, TILE_GROUND=2, TILE_CLIFF=3 };


SampleEventHandler::SampleEventHandler(SampleAIState &state_, sint4 seed) : state(state_) {     
  minerals.clear();
  minerals.clear();
  controlCenter = NULL;
  rand.srand(seed);
}


bool SampleEventHandler::handle_event(const Event& e)
{

  cout << "Event: " << e.get_who() << " " << e.get_what() << endl;
  if (e.get_who() == GameStateModule::FROM) {

    //cout << "Event: " << e.get_who() << " " << e.get_what() << endl;

    if (e.get_what() == GameStateModule::STOP_MSG) {
      state.quit = true; return true;
    }

    if (e.get_what() == GameStateModule::READ_ERROR_MSG) {
      state.quit = state.error = true; return true;
    }

    if (e.get_what() == GameStateModule::VIEW_MSG) {

      compute_actions();
      state.gsm->send_actions();

      Game &game = state.gsm->get_game();	

      // gfx and frame statistics

      if (state.gfxm) state.gfxm->process_changes();

      sint4 vf = game.get_view_frame();
      sint4 af = game.get_action_frame();
      sint4 sa = game.get_skipped_actions();
      sint4 ma = game.get_merged_actions();

      if ((vf % 20) == 0) cout << "[frame " << vf << "]" << endl;

      if (vf != af || sa || ma > 1) {

        cout << "frame " << vf;
        if (af < 0) cout << " [no action]";
        else if (vf != af) cout << " [behind by " << (vf - af) << " frame(s)]";
        if (sa) cout << " [skipped " << sa << "]";
        if (ma > 1) cout << " [merged " << ma << "]";
        cout << endl;
      }

      if (state.gfxm) {

        // don't draw if we are far behind
        if (abs(vf-af) < 10) {
          state.gfxm->draw();
          state.just_drew = true;
        }
      }

      if (state.gui) {
        state.gui->event();
        //        state.gui->display();
        if (state.gui->quit) exit(0);
      }

      // as a hack for flickering, gui->display() is called in dot.start()
      // so call everything that draws to the debug window between start() and end()
      //DrawOnTerrain2D dot(state.gui);
      //dot.start();
      //dot.draw_line(-60, 10, -20, 10, Vec3<real4>(1.0, 1.0, 1.0));
      //dot.draw_line(-60, 10, -65, 15, Vec3<real4>(1.0, 1.0, 1.0));
      //dot.draw_line(-44, 10, -56, 40, Vec3<real4>(1.0, 1.0, 1.0));
      //dot.draw_line(-28, 10, -36, 40, Vec3<real4>(1.0, 1.0, 1.0));
      //dot.draw_line(-36, 40, -30, 34, Vec3<real4>(1.0, 1.0, 1.0));

      //dot.draw_circle(-40, 25, 35, Vec3<real4>(1.0, 0.0, 1.0));

      //dot.end();

      return true;
    }
  }
  return false;
}


void SampleEventHandler::compute_actions()
{

  sint4 currentSpeed = 0;
  cout << "compute actions" << endl;
  const GameChanges &changes = state.gsm->get_changes();

  //  sets init positions of minerals and command center
  for(typeof(changes.new_objs.begin()) it = changes.new_objs.begin(); it != changes.new_objs.end(); it++){
    GameObj* gob=(*it)->get_GameObj();

    if(gob->bp_name() == "worker"){
      workers.push_back(LocalWorker(gob));
    }
    else if(gob->bp_name() == "mineral"){
      minerals.insert(gob);
    }
    else if(gob->bp_name() == "sheep"){
      sheeps.insert(gob);
    }
    else if(gob->bp_name() == "controlCenter"){
      controlCenter=gob;
    }
  }

  // minerals are emppty
  for(typeof(changes.dead_objs.begin()) it = changes.vanished_objs.begin(); it != changes.vanished_objs.end(); it++){
    GameObj *gob=(*it)->get_GameObj();
    if(gob->bp_name( )== "mineral"){
      minerals.erase(gob);
    }
  }

  cout << "Workers " << endl;
  for (std::vector<LocalWorker>::iterator it = workers.begin(); it != workers.end(); it++){
    printGemeObjectInfo((*it).getGOB());
    //cout << << endl;
  }

  //cout << "Minerals " << endl;
  //for (std::set<GameObj*>::iterator it = minerals.begin(); it != minerals.end(); it++){
  //  printGemeObjectInfo(*it);
  //  //cout << << endl;
  //}

  //cout << "Sheeps " << endl;
  //for (std::set<GameObj*>::iterator it = sheeps.begin(); it != sheeps.end(); it++){
  //  printGemeObjectInfo(*it);
  //  //cout << << endl;
  //}
  printGemeObjectInfo(controlCenter);

  // process oly 1
  process(&(*workers.begin()));

  // process all
  //for (std::vector<LocalWorker>::iterator it = workers.begin(); it != workers.end(); it++){
  //  process(&(*it));
  //}

  //// ***************** objects ******************
  //// units under client player control

  //const Game &game = state.gsm->get_game();

  //const sint4 cid = game.get_client_player();
  //const sint4 n = game.get_player_num();

  //// objects under client player control
  //const Game::ObjCont &opp_objs = game.get_objs(cid);
  //// neutral objects
  //const Game::ObjCont &n_objs = game.get_objs(n);

  //for (typeof(opp_objs.begin()) it1 = opp_objs.begin(); it1 != opp_objs.end(); it1++){

  //    GameObj *gob = (*it1)->get_GameObj();

  //    // debug info
  //    printGemeObjectInfo(gob);

  //    if ((gob->bp_name()== "worker")&&(gob->get_speed(&currentSpeed)==0)){
  //        for (typeof(n_objs.begin()) it2 = n_objs.begin(); it2 != n_objs.end(); it2++){

  //            GameObj *gob_n = (*it2)->get_GameObj();
  //            if (gob_n->bp_name()== "mineral"){

  //                sint4 target_x = *gob_n->sod.x;
  //                sint4 target_y = *gob_n->sod.y;

  //                Vector<sint4> params;
  //                params.push_back(target_x);
  //                params.push_back(target_y);

  //                gob->set_action("move", params);
  //            }
  //        }
  //        //break;

  //    }
  //}
}


void SampleEventHandler::printGemeObjectInfo(GameObj *gob){

  const ServerObjData &sod = gob->sod;


  cout << "bp: " << gob->bp_name();

  if (!sod.in_game) { 
    cout << " no gameobj" << endl;
    return;
  }

  if (*sod.x < 0) {
    cout << "not on playfield" << endl;
    return;
  }

  // object on playfield

  //Vector<std::string> attr_names;
  //gob->attr_names(attr_names);

  //cout << "||| ";
  //for(Vector<std::string>::iterator it = attr_names.begin(); it != attr_names.end(); it++){
  //    cout << *it << " - ";
  //}

  //Vector<std::string> component_names;
  //gob->component_names(component_names);
  //
  //cout << "||| ";
  //for(Vector<std::string>::iterator it = component_names.begin(); it != component_names.end(); it++){
  //    cout << *it << " - ";
  //}


  cout << " zcat: " << *sod.zcat
    << " max-speed: " << *sod.max_speed
    << " speed: " << *sod.speed 
    << " moving: " << *sod.is_moving 
    << " sight: " << *sod.sight
    << " X : " << *sod.x
    << " Y : " << *sod.y;

  if (*sod.shape == Object::CIRCLE) {
    cout << " CIRCLE loc: (" << *sod.x << "," << *sod.y << ")"
      << " r: "<< *sod.radius;

  } else if (*sod.shape == Object::RECTANGLE) {
    cout << " RECT dim: (" << *sod.x1 << "," << *sod.y1 << ")-("
      << *sod.x2 << "," << *sod.y2 << ")";
  } else {
    cout << " UNKNOWN SHAPE";
  }

  // - other object properties

  // heading: 0 = stopped, 1...GAME_CONST::HEADING_N
  const sintptr *hd = gob->get_int_ptr("heading");
  if (hd){ 
    cout << " hd: " << GameConst::angle_from_dir(*hd, GameConst::HEADING_N);
  }

  cout << endl;
  // ... more examples

}


void SampleEventHandler::process(LocalWorker* lw){

  const Game &game = state.gsm->get_game();
  Vector<sint4> params;
  
  if ((*(lw->getGOB()->sod.speed) == 0)&&!(lw->getGOB()->is_pending_action())){
    switch (lw->getState()){
      case IDLE:
        assignMineral(lw);
        lw->moveToTargetMineral();
        lw->setState(WALKING_TO_MINERAL);
        cout << "----------go to Mineral" << endl;
        break;
      case WALKING_TO_MINERAL:
        if (Distance::distance(*(lw->getGOB()), *(lw->getTargetMineral())) > 10){
          lw->setTX(*(lw->getGOB()->sod.x) - 20 + std::rand() % 40);
          lw->setTY(*(lw->getGOB()->sod.y) - 20 + std::rand() % 40);
          lw->moveToXY();
          lw->setState(WALKING_TO_MINERAL_STUCK);
          cout << "----------I am IDLE in the way to mineral" << endl;
        }
        else{
          params.clear();
          params.push_back(game.get_cplayer_info().get_id(lw->getTargetMineral()));
          lw->getGOB()->component("pickaxe")->set_action("mine", params);
          lw->setState(MINING);
          cout << "----------Start MINING " << endl;
        }
        break;
      case WALKING_TO_MINERAL_STUCK:
        if(Distance::distance(*(lw->getGOB()), lw->getTX(), lw->getTY()) > 5){
          if (lw->getAttempt() == 10 ){
            lw->setState(IDLE);
            cout << "----------I am tring to reach new mineral " << endl;
          }else{
            lw->setTX(*(lw->getGOB()->sod.x) - 20 + std::rand() % 40);
            lw->setTY(*(lw->getGOB()->sod.y) - 20 + std::rand() % 40);
            lw->moveToXY();
            cout << "----------I am tring new point to aproach mineral " << endl;
          }
        }
        else{
          lw->moveToTargetMineral();
          lw->setState(WALKING_TO_MINERAL);
          cout << "----------go to Mineral again" << endl;
          }
        break;
      case MINING:
        if(lw->getGOB()->component("pickaxe")->get_int("active") == 0){
          lw->moveTo(*(controlCenter->sod.x), *(controlCenter->sod.y));
          lw->setState(WALKING_TO_CONTROL_CENTER);
          cout << "----------go to conrol center " << endl;
        }else{
          cout << "---------- MINING " << endl;
        }
        break;
      case WALKING_TO_CONTROL_CENTER:
        if(Distance::distance(*(lw->getGOB()), *controlCenter) > 5){
          lw->setTX(*(lw->getGOB()->sod.x) - 20 + std::rand() % 40);
          lw->setTY(*(lw->getGOB()->sod.y) - 20 + std::rand() % 40);
          lw->moveToXY();
          lw->setState(WALKING_TO_CONTROL_CENTER_STUCK);
          cout << "----------I am IDLE on my way to comand center" << endl;
        }else{
          params.clear();
          params.push_back(game.get_cplayer_info().get_id(controlCenter));
          lw->getGOB()->set_action("return_resources", params);
          lw->setState(RETURNING_RESOURCES);
          cout << "----------return resources " << endl;
        }
        break;
      case WALKING_TO_CONTROL_CENTER_STUCK:
        if(Distance::distance(*(lw->getGOB()), lw->getTX(), lw->getTY()) > 5){
          lw->setTX(*(lw->getGOB()->sod.x) - 20 + std::rand() % 40);
          lw->setTY(*(lw->getGOB()->sod.y) - 20 + std::rand() % 40);
          lw->moveToXY();
          cout << "----------I am tring new point" << endl;
        }else{  
          lw->moveTo(*(controlCenter->sod.x), *(controlCenter->sod.y));
          lw->setState(WALKING_TO_CONTROL_CENTER);
          cout << "----------I am going to control center" << endl;
        }
        break;
      case RETURNING_RESOURCES:
        if(lw->getGOB()->get_int("minerals") == 0 ){
          lw->setState(IDLE);
          cout << "----------I am IDLE from returning minerals" << endl;
        }else{
          cout << "----------I am still returning minerals" << endl;
        }
        break;
      default: 
      cout << "----------Problem!!!" << endl;
    }
  }else{
      cout << "---------- empty operation" << endl;
  }

  cout << "-----------------worker state "<< lw->getState() 
    << " pending: "<< lw->getGOB()->is_pending_action() 
    << " Minerals: "<< lw->getGOB()->get_int("minerals") 
    << endl;
}
void SampleEventHandler::assignMineral(LocalWorker* lw){
 
  int i;
  int r;
  set<GameObj*>::size_type b;  
  set<GameObj*>::iterator it;

  b = minerals.size();
  r = std::rand() % static_cast<int>(b);
  for (i = 0, it = minerals.begin(); i < r ; i++, it++){}
  lw->setTargetMineral(*it);
  lw->setAttempt(0);
}

