#ifndef SampleEventHandler_H
#define SampleEventHandler_H

// $Id: SampleEventHandler.H,v 1.1 2007/04/25 20:40:29 shstoyko Exp $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

#include "EventHandler.H"
#include "Random.H"
#include "LocalWorker.h"


class GameStateModule;
class GfxModule;
class GUI;

class GameObj;

using std::vector;
using std::set;

// This class gives the event handler access to all game and gfx data

struct SampleAIState {

  GameStateModule *gsm;
  GfxModule       *gfxm; // !=0 : gfx used
  GUI             *gui;  // !=0 : debug gfx used
  bool just_drew;  // just drew a gfx frame
  bool quit;       // true => quit game
  bool error;      // true <=> error occurred
 
  SampleAIState(GameStateModule *gsm_, GfxModule *gfxm_, GUI *gui_) :
    gsm(gsm_), gfxm(gfxm_), gui(gui_)
  {
    just_drew = quit = error = false;
  }

};


// This class handles server events
// In particular it generates game object actions

class SampleEventHandler : public EventHandler {

public:  
  SampleAIState &state;
  SampleEventHandler(SampleAIState &state, sint4 seed); 
  bool handle_event(const Event &e);

private: 
  vector<LocalWorker> workers;
  set<GameObj*> minerals;
  set<GameObj*> sheeps;
  GameObj* controlCenter;
  //WorkerController wc;
  void compute_actions();
  void printGemeObjectInfo(GameObj *gob);
  void process(LocalWorker* lw);
  void assignMineral(LocalWorker* lw);
  Random rand; // local RNG
  
};

#endif
