#ifndef Handler_H
#define Handler_H

// $Id: Handler.H,v 1.1.1.1 2007/04/09 19:05:50 pjungkun Exp $

#define QT_THREAD_SUPPORT

#include "Global.H"

#include <qthread.h> 

#include <qobject.h> 
#include <qwidget.h>
#include <qapplication.h>

#include "Net_GGSClient.H"
#include "Net_Protocol.H"
#include "Net_Message.H"
#include "MessageEvent.H"
using namespace std;
using namespace Protocol;
 
typedef __gnu_cxx::hash_map<int, pair<Invit_Disc, uint1> > InvitationsList;
typedef __gnu_cxx::hash_map<string, uint1, StringHash>     BuddyList;
typedef __gnu_cxx::hash_map<uint4, pair<string, uint1 > >  PlayerList;
typedef __gnu_cxx::hash_map<string, uint1, StringHash>     WhoList;
typedef __gnu_cxx::hash_map<string, string, StringHash>    ChatContextList;

REGISTER_TYPEOF(70, InvitationsList);
REGISTER_TYPEOF(71, BuddyList);
REGISTER_TYPEOF(72, ChatContextList);


class Handler : public QThread {
public:
  
  Handler(const string &server_, sint4 port_, QObject *receiver_);
  
  virtual ~Handler();
  
  void tellService    (const string &message);
  void tellPeer   (const string &peer, const string &message);
  void tellBuddies(const string &message);
  void tellGGS    (const string &message);
  void tellContext(const string &context);
  /**
     logs onto the service using the given pass and level
  */
  void login(const string &nick_, const string &pass_, Level level_, const string &server_ip, const string &port_, const string &servicename_);
  void GGSlogin(const string &nick_, const string &pass_, const string &server_ip, const string &port_, const string &servicename_);
  void run();
  
  
  // list initializers
  void initHandler();
  
  // accessors :
  const InvitationsList& getInvitations()  const { return invitations; }
  const BuddyList&       getBuddies()      const { return buddyList;   }
  const WhoList&         getUsers()        const { return whoList;     }
  const PlayerList&      getPlayers()      const { return playerList;  }
  int get_iid() const { return current_iid; }
  int get_pid() const { return current_pid; }
  
  void createContext(const string &to);
  void deleteContext(const string &to);
  
  string get_workerinfo() { return current_info; }
  string get_server() { return server; }
  
private:
  
  void handle_chat        (const Net_Message    &m);
  void handle_loginNotify (const Login_Notify   &notify);
  void handle_game_start  (const Game_Start     &start);
  void handle_game_info   (const Game_Info      &info);
  void handle_invitations (const Invitations    &inv);
  void handle_aborted     (const Aborted        &aborted);
  void handle_buddies     (const Buddies        &buddies);
  void handle_joined      (const Joined         &joined);
  void handle_playerList  (const Player_List    &players);
  void handle_whoList     (const Who_List       &who);
  void handle_left        (const Protocol::Left &left_);
  void handle_readied     (const Readied        &readied);
  void handle_unreadied   (const Unreadied      &unreadied);
  void handle_added       (const Added_Buddy    &added_buddy);
  void handle_removed     (const Remed_Buddy    &remed_buddy);
  void handle_kicked      (const Kicked         &kicked);
  void handle_tell        (const Tell           &tell);
  void handle_error       (const Error          &error);
  
  bool awaitResponce(Message_Base& expected);
  
  void clearInGameInfo();
  
  InvitationsList invitations;
  BuddyList       buddyList;
  PlayerList      playerList;
  WhoList         whoList;
  ChatContextList contextList;
  
  Net_GGSClient* client;
  
  string nick; 
  string pass;
  string server;
  sint4  port;
  Level level;
  QObject *receiver;
  string servicename;
  
  int current_iid;   ///< iid of the currently joined game
  int current_pid;   ///< pid in the currently joined game
  string current_info;
  
  // status booleans
  bool loginOK;
  bool buddyListOK;
  bool inInvitation;
  bool usersListOk;
};

#endif
