// $Id: Net_Invitation.H,v 1.1.1.1 2007/04/09 19:05:53 pjungkun Exp $

// (c) Michael Buro 2003, mburo@cs.ualberta.ca, licensed under the GPL

/**
 * An object that holds an invitation
 */
	
#ifndef Net_Invitation_H
#define Net_Invitation_H

#include "Global.H"
#include "Net_Protocol.H"

class Net_Invitation
{

public:

  /**
   * \brief does nothing
    * for the default call by derived classes
    */
  Net_Invitation(){}
  Net_Invitation(Invite invite);
  Net_Invitation(const Net_Invitation& n);
  virtual ~Net_Invitation();

  /** returns the info if the current game */
  const std::string& getInfo() const { return info; }

  /** returns the iid of this invitation */
  sint4 getIID() const { return iid; }
  std::string getOwner() const { return owner; }
  std::string getPass() const { return password; }
  bool getAnony() const { return anony_ids; }
  bool getInvisible() const { return invisible; }
  
  const Vector<std::string>& getAliases() const { return aliases; }
  const Vector<std::string>& getPlayers() const { return players; }
  const Vector<uint1>& getPresent() const  { return present; }
  const Vector<uint1>& getReady  () const  { return   ready; }
    
  void setIID(uint4 iid_);
  
  /**
    sets ready to ready_ (true = ready, false = unready)
    @return the iid of the player
  */
  int setReady(const std::string& name, bool ready_);
  
  sint4 addPlayer(const std::string& name, const std::string& alias);

  /**
    removes a player
    @return player alias
  */
  sint4 removePlayer(const std::string& name);

  /**
    removes a player
    @return player alias
  */
  sint4 removePlayer(sint4 pid);

  uint4 getPlayerCount() const { return playerCount; }

  bool allReady() const { return readyCount == maxPlayers; }
  //const std::string& getStd::String(){ return inviteStd::String; }
  
  void print();

  std::string getOptions() const { return options; }
  
protected:
  /** updates the info stirng of the current game */
  void updateInfo();

  std::string owner;
  std::string name;
  bool invisible;
  bool anony_ids;
  bool passprotected;
  std::string alias;
  std::string password;
  std::string options;

  std::string info;
  
  uint4 readyCount;
  uint4 playerCount;
  uint4 maxPlayers;
  
  uint4 iid;
  Vector<std::string> aliases;
  Vector<std::string> players;
  Vector<uint1>  ready;
  Vector<uint1>  present; /** to keep whether a player is present or not... to avoid using NULL pointers */
  
  /**
    searches linearly for the player
    returns players index
  */
  sint4 getPlayer(const std::string& name);
};

#endif

