// $Id: Net_Worker.C,v 1.1.1.1 2007/04/09 19:05:53 pjungkun Exp $

// This is an ORTS file (c) Tu Hoang, licensed under the GPL

/*
 * ====================================================================================
 * 
 *        Filename:  Net_Worker.C
 * 
 *     Description:  main files
 * 
 *         Version:  1.0
 *         Created:  05/25/04 15:39:07 MDT
 *        Revision:  none
 *        Compiler:  gcc
 * 
 *          Author:  Tu Hoang 
 * 
 * ====================================================================================
 */

#include "Net_Worker.H"
#include "Options.H"

using namespace std;

extern void create_orts_world(stringstream &w);

Net_Worker::Net_Worker(const string &world_)
{
  signals_shut_down(false);
  world = world_;  
}


Net_Worker::~Net_Worker()
{
  if (client) delete client;
}


bool Net_Worker::handle_event(const Event& e)
{
  if (!client) { 
    cerr << "ERROR: {Net_Worker::handle_event}: no GGS client make sure run Net_Worker::run first.\n";  
    exit(-1);
  }
  
  if (e.get_who() == Server::FROM && e.get_what() == Server::READY_MSG ){
    REM("------ Notify Service: worker ready"); 
    client->send("tell " + servicename + " " + Worker_Ready::compose());
    return 1;
  }
 
  return 0;
}


sint4 Net_Worker::run()
{
  // message parsers
  Finger finger;
  
  string server_ip;
  sint4 sport;
  
  Options::get("-username", user_name);
  Options::get("-password", password);
  Options::get("-server", server_ip);
  Options::get("-sport", sport);
  Options::get("-servicename", servicename);
  
  client = new Net_GGSClient(user_name, password, server_ip, sport, true);
  string message;
  Net_Message net_msg;

  sint4 port = 0;
  Options::get("-port", port);

  // get IP
  string hostIP;

  client->send(Finger::compose(user_name));
  bool ipOk = (waitForResponse(finger) && finger.get("host") != "NULL");

  if (!ipOk) { ERR("unable to retrieve local host ip"); }

  hostIP = finger.get("host");

  /***************************************************/
  // GGS inserts 0's to make all three sections of ip addresses
  // three chars long... i donno why
  // Ex: 123.124.22.65 -> 123.124.022.065
  //                              ^   ^
  // this cannot be used as an address to 
  // connect to so it must be cleaned up
  
  stringstream cleanIP;

  FORU (i, hostIP.length()){
    if (hostIP[i] == '.') hostIP[i] = ' ';
  }

  stringstream ss(hostIP);
  int tint;
  ss >> tint;

  FOREVER {
    cleanIP << tint;
    if(!(ss >> tint)){ break; } // skip the last dot
    cleanIP << ".";
  }

  if (ss.bad()){
    ERR("could not clean up IP");
  } else {
    hostIP = cleanIP.str();
  }

  /****************************************************/

  // login into the service
  string send_msg = "tell " + servicename + " " + Login::compose(WORKER, password, hostIP, port) ;
  client->send (send_msg);

  Login_Notify notify;
  bool loginOK = waitForResponse(notify) && notify.get_player_name() == user_name;
  
  if (!loginOK) {
    ERR("Unable to login to the Service");
  } else {
    REM("login successful");
  }

  Worker_Start start;
  
  while (!shut_down) {

    REM("Waiting for START command");
    waitForResponse(start);
    if (start.get_sender() != servicename) continue;
    REM("START command received");

    if (Options::process(start.get_options(), cerr, "version ")) {
      REM2("\nInvalid options : " , start.get_options());
      continue;
    }

    REM2("Options : ", start.get_options());

    if (Options::process(start.get_options(), cerr, "started as ortsnet worker"))
      exit(20);

    stringstream world;
    create_orts_world(world);

    Server server;

    server.add_handler(this); // server calls handle_event each frame
    
    // todo: call server.add_login_info(...) with client login information
    
    Server::Options so;
    server.run(world.str(), so);

    // todo: send out game result to players and service
    
    // tell the service this worker is available to start playing a new game
    string msg = "tell " + servicename + " " + Worker_Available::compose(hostIP);
    client->send(msg);

    // ~Server() disconnects clients and deletes game
  }

  return 0;
}


bool Net_Worker::waitForResponse(Message_Base& expected)
{
  string message;
  Net_Message m;

  while (!shut_down) {
    client->recv(message);
    cout << message << endl;
    m.setMessage(message);
    if (expected.parse(m)) return true;
  }

  return false;
}
