// $Id: nptest_main.C,v 1.1.1.1 2007/04/09 19:05:54 pjungkun Exp $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

// this is the main file of the sample ORTS AI client
// options are specified by the opt.put lines below
// to test it start the ORTS server (bin/orts) and then run bin/sampleai [options]

// NP_EventHandler.C contains the code for accessing the game state

#include "GfxGlobal.H"
#include "Options.H"
#include "GfxModule.H"
#include "GameStateModule.H"
#include "ShutDown.H"
#include "SDLinit.H"
#include "Game.H"

#include "SimpleTerrain.H"
#include "TerrainModule.H"
#include "NP_EventHandler.H"

static char *VERSION = "nptest v0.1";

using namespace std;

static bool just_drew = false;

// loop is called

class Looper : public LoopFunctor
{
public :

  Looper(SampleAIState *state_) : state(state_) {}
  
  void loop() const {

    if (state->gfxm) {

      if (state->gfxm->get_quit()) { state->quit = true; exit(0); } // should return

      bool refresh;
      Options::get("-refresh", refresh);
      
      if (refresh) {

	// maximal gfx refresh rate
	
	if (!just_drew) 
	  state->gfxm->draw();
	else
	  just_drew = false;
      }
    }
  
    // looks for server messages
    // if one or more arrived, calls handlers
    // if not, sleeps for one millisecond
    if (!state->gsm->recv_view()) SDL_Delay(1);
  }

private:
  SampleAIState *state;
};


int main(int argc, char **argv)
{
  SimpleTerrain::ST_Terrain terr;
 
  glutInit(&argc, argv);

  signals_shut_down(true);

  Options opt("SampleClient");
  opt.put("-gfx", "display world in 3d");
  opt.put("-cinfo", "display game state changes");
  opt.put("-tinfo", "display tile info");
  opt.put("-oinfo", "display object info");
  opt.put("-minfo", "display map info");    
  
  // populate opt with options from various modules
  
  GameStateModule::Options::add();
  GfxModule::Options::add();
  SimpleTerrain::ST_Terrain::add_options();

  // process command line arguments
  if (Options::process(argc, argv, cerr, VERSION)) exit(20);

  // initialize SDL
  SDLinit::video_init();
  SDLinit::network_init();
  
  // populate option objects with command line values
  // and create modules
  
  GameStateModule::Options gsmo;
  GameStateModule gsm(gsmo);
  TerrainModule tm(gsm, terr);

  GfxModule::Options gfxmo;
  GfxModule *gfxm = 0;

  // change seed value to time(0) if 0

  sint4 seed;
  Options::get("-seed", seed);

  if (!seed) seed = time(0);

  Options::set("-seed", seed);
  cout << "seed=" << seed << endl;

  bool use_gfx;
  Options::get("-gfx", use_gfx);

  if (use_gfx){
    gfxm = new GfxModule;
    gfxm->add_handler(&tm);
  }

  // gives event handler access to the state
  SampleAIState state(&gsm, gfxm);

  SampleEventHandler sev(state, seed);

  // gsm events are handled in sev
  gsm.add_handler(&sev);
  //gsm.add_handler(&tm); done in TerrainModule
  // connect game state module to server
  if (!gsm.connect()) ERR("connection problems");

  Looper l(&state);

  if (use_gfx) {

    // gfx mode

    gfxm->init(gsm.get_game(), gsm.get_changes(), gsm.get_action_changes(), gfxmo);

    // game loop (could become a thread)
    gfxm->start_loop(&l);

  } else {

    // no gfx
    
    while (!state.quit) l.loop();
  }
  
  delete gfxm;
  return 0;
}
