// $Id: osl_test_main.C,v 1.1.1.1 2007/04/09 19:06:19 pjungkun Exp $ 

// (c) Jonathan Yip, licensed under the GPL


#include "OSL.H"
#include "OSL_Opcodes.H"
#include "OSL_RecStream.H"
#include "Lexer.H"
#include "OSL_Parser.H"

using namespace std;
using namespace OSL;

void lexer_test()
{
  RecStream rs;
  Lexer lexer(rs);
  const string fname = "test/lexer-test.osl";
  FILE *fp = fopen(fname.c_str(), "r");

  if (!fp) ERR("file not present: "+ fname);
  
  rs.push(fp, fname);
 
  FOREVER {

    Token tok;
    
    try {
      tok = lexer.get_token();
      tok.write(); cout << " ";
      if (tok.get_type() == TK_EOF) break;
    }
    catch (Lexer::SyntaxErrorException &e) {
      cout << "Syntax error: " << e.msg << endl;
      break;
    }
    catch (RecStream::ReadErrorException &e) {
      cout << "Read error: " << e.msg << endl;
      break;
    }
    
  }

  cout << endl;
}


void vm_loop_test()
{
  Program prog;
  Context cont(prog);
  FuncMember fm;

  // loops from 1 to 10 (sint4) and prints all values

  fm.add_code(Opcodes::PUSH_ZS4);
  CodeAddr loop = fm.get_addr();
  fm.add_code(Opcodes::PUSH_CS4);
  fm.add_sint4(1);
  fm.add_code(Opcodes::ADD_SS4);
  //  fm.add_code(Opcodes::DUP);
  //  fm.add_code(Opcodes::PRINT);
  fm.add_code(Opcodes::DUP);
  fm.add_code(Opcodes::PUSH_CS4);
  //  fm.add_sint4(4000000);
  fm.add_sint4(4);
  fm.add_code(Opcodes::LT_SS4);
  fm.add_code(Opcodes::JUMP_TRUE);
  fm.add_addr(loop);
  fm.add_code(Opcodes::RETURN_VAL);

  prog.write_code(fm, cout);

  vector<Value> params;
  cont.init_run(&fm, params);
  cont.run();

  Value v = cont.result();

  prog.write_value(v);
}


Value script_print(vector<Value> &params) {
  uint4 end = params.size();
  cout << "Inside c++ " << endl;
  FORU(i, end) Program::write_value(params[i]);
  cout << "Leaving c++ " << endl;
  Value v;
  return v;
}


int main()
{
  //  lexer_test();
  //  vm_loop_test();

  cout << "\n*****\nCompiler test\n*****" << endl;
  Program prog;
  vector<TypeIndex> par_types;
  par_types.push_back(Type::TC_sint4);
  prog.add_ext(script_print, "print", par_types);
  Parser p(prog, "test/parser-test.osl");
  //Parser p(prog, "test/lexer-test.osl");
  p.parse();

  cout << "Successful parse" << endl;

  // run the main function from global class
  vector<Value> params;
  Value v = prog.run("two", "fn", params);
  prog.write_value(v);

  cout << "DONE" << endl;

#if 0
  uint4 index = prog.class_ids["abc"];
  prog.write_code(prog.classes[index].virt_funcs[0], cout);

  Context cont(prog);
  vector<Value> params;
  cont.init_run(&prog.classes[index].virt_funcs[0], params);
  cont.run();
  Value v = cont.result();
  prog.write_value(v);
#endif

  return 0;
}
