#ifndef MyPathfindEnvironment_H
#define MyPathfindEnvironment_H

// $Id: MyPathfindEnvironment.H,v 1.1.1.1 2007/04/09 19:06:46 pjungkun Exp $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

#include"pathfind.h"
#include"PolyGraph.H"
#include "Coord.H"

/** Description

    Code for pathfinding in the AI
*/

#define node_id_type int

class MyPathfindEnvironment: public PathFind::Environment
{
  const PolyGraph* graph;

  node_id_type m_start;
  node_id_type m_goal;
    
public:
  MyPathfindEnvironment();
  ~MyPathfindEnvironment(){}
    
  void attach_graph(const PolyGraph*);
  void detach_graph();
    
  node_id_type point_to_id(const ScalarPoint &pt) const;
  Point2i id_to_point(node_id_type id) const;
    
  void set_start(node_id_type id);
  void set_goal(node_id_type id);

  int getCost(node_id_type source, node_id_type target) const;
    
  // PathFind::Environment methods
  virtual int getHeuristic(node_id_type start, node_id_type target) const;
  virtual int getMaxCost() const;
  virtual int getMinCost() const;
  virtual int getNumberNodes() const;

  //cost between two nodes
    
  /** Generate successor nodes for the search.
      @param nodeId Current node
      @param lastNodeId
      Can be used to prune nodes,
      (is set to NO_NODE in Search::checkPathExists).
      @param result Resulting successors.
      Passed in as a reference to avoid dynamic memory allocation.
      The vector will be cleared and filled with the successors.
  */
  virtual void getSuccessors(node_id_type nodeId, node_id_type lastNodeId,
			     std::vector<Successor>& result,
			     bool usePruning = false) const;
    
  virtual bool isValidNodeId(node_id_type nodeId) const;
};

#endif
