// $Id: ShooterAI.C,v 1.1.1.1 2007/04/09 19:06:47 pjungkun Exp $

// This is an ORTS file (c) Michael Buro, Tim Furtak licensed under the GPL

// shooter "ai": attack everything within range

#include "ShooterAI.H"
#include "GameStateModule.H"
#include "Game.H"
#include "GameObj.H"
#include "GameChanges.H"
#include "Profiler.H"

using namespace std;

const std::string ShooterAI::bp_name = "shooter";

//===================================================================

ShooterAI::ShooterAI(GameStateModule &gsm_) : gsm(gsm_)
{
  enabled = false;
}

//-------------------------------------------------------------------

ShooterAI::~ShooterAI()
{
}

//===================================================================

GameObj* ShooterAI::select_target(const Vector<GameObj*> &targets)
{
  if (targets.empty()) return 0;
  return targets[0];
}

//===================================================================

void ShooterAI::act(GameStateModule &/*gsm*/)
{
  if (!enabled) return;

  Game &game = gsm.get_game();

  FORALL (friendly_data, it) {
    GameObj *shooter = it->first;
    const ShooterData &sd = it->second;
    GameObj *targ = select_target(sd.in_range);
    if (!targ) continue;

    Vector<sint4> args;
    args.push_back(game.get_cplayer_info().get_id(targ));
    shooter->component("weapon")->set_action("attack", args);
  }
}

//===================================================================

bool ShooterAI::handle_event(const Event &e)
{
  if (e.get_who() == GameStateModule::FROM) {
    
    //cout << "Event: " << e.who() << " " << e.what() << endl;
    
    if (e.get_what() == GameStateModule::VIEW_MSG) {
      update(gsm.get_changes());

      // compute actions
      act(gsm);
    }
  }

  return false;
}

//===================================================================

void ShooterAI::update(const GameChanges &changes)
{
  sint4 me = gsm.get_game().get_client_player();
  sint4 npc = gsm.get_game().get_player_num();

  FORALL (changes.new_objs, i) {
    GameObj *gob = (*i)->get_GameObj();
    assert(gob);
    sint4 owner = *gob->sod.owner;
    if (owner == me)       add_friendly(gob);
    else if (owner != npc) enemies.insert(gob);
  }

  FORALL (changes.vanished_objs, i) {
    GameObj *gob = (*i)->get_GameObj();
    assert(gob);
    sint4 owner = *gob->sod.owner;
    if (owner == me)       friendlies.erase(gob);
    else if (owner != npc) enemies.erase(gob);
  }

  FORALL (changes.dead_objs, i) {
    GameObj *gob = (*i)->get_GameObj();
    assert(gob);
    sint4 owner = *gob->sod.owner;
    if (owner == me)       friendlies.erase(gob);
    else if (owner != npc) enemies.erase(gob);
  }

  recompute_targets();
}

//===================================================================

void ShooterAI::add_friendly(GameObj *gob)
{
  ScriptObj *weapon = gob->component("weapon");
  if (!weapon) return;
  friendlies.insert(gob);
}

//===================================================================

void ShooterAI::recompute_targets()
{
  PROFILE("Shooter - compute targets");
  if (!enabled) return;

  Game &game = gsm.get_game();

  friendly_data.clear();
  enemy_data.clear();

  FORALL (friendlies, f) {

    ScriptObj *weapon = (*f)->component("weapon");
    const sintptr *r = weapon->get_int_ptr("max_ground_range");
    if (!r) continue;
    sint4 range = *r;

    ShooterData sd;

    FORALL (enemies, e) {
      real8 rdist = game.distance(**f, **e);
      sint4 sdist = (sint4)floor(rdist);
      if (sdist > range) continue;
      
      //      cout << "in range - " << range << "   " << sdist << endl;

      bool can_see = game.can_see_obj(**f, **e);
      if (!can_see) continue;
      
      sd.in_range.push_back(*e);
      enemy_data[*f].in_range.push_back(*f);
    }

    friendly_data[*f] = sd;
  }
}

//===================================================================

