#include "GAISquad.H"
#define pi 	3.14159265

using namespace std;

GAISquad::GAISquad(int numberOfTanks,bool lft) {
	left = lft;
	squad_tanks = new GAITank*[numberOfTanks];
	oppBases = new GameObj*[5];
	index = 0;
	tanks = numberOfTanks;
	firstTime = 1;
	currentPosition.x = -1;
	currentPosition.y = -1;
	squad_formed = 0;
	pathfinder = new aStarLibrary();
	pathfinder->InitializePathfinder();
}

void GAISquad::addTank(GAITank* tankje) {
	squad_tanks[index] = tankje;
	index++;
	if ( currentPosition.x < 0 ) {
		currentPosition = tankje->getPosition();
	}
}

bool GAISquad::actPerTank(const Game& game) {
	FORS ( it, tanks ) {
		squad_tanks[it]->act(game);
	}
	return true;
}

//int aStarLibrary::mapMade;

bool GAISquad::act(const Game& game) {
	if ( firstTime ) {
		doOnce = 1;
		if ( pathfinder->mapMade == 0 ) {
			// make the initial walkabilitymap
			const Map<GameTile> &map = game.get_map();
			for (int x = 0; x <= 63; x++){
				for (int y = 0; y <= 63; y++){
					pathfinder->walkability [x][y] = GAISquad::TileWalkable(map(x,y));
				}
			}
			pathfinder->mapMade = 1;
		}
		// init mywalkability
		for ( int i = 0; i < 64; i++ ) 
			for ( int j = 0; j < 64; j++ ) 
				myWalkability[i][j] = pathfinder->walkability[i][j];
		
		// adjust to squad width	
		const Map<GameTile> &mapTemp = game.get_map();
		for ( int i = 0; i < 64; i++ ) {
			for ( int j = 0; j < 64; j++ ) {
				if ( pathfinder->walkability[i][j] == pathfinder->unwalkable  && GAISquad::TileWalkable(mapTemp(i,j)) == pathfinder->unwalkable)
				{
					for ( int k = -2; k <= 2; k++ )
						for ( int m = -2; m <= 2; m++ )
							if ( i+k >= 0 && i+k <= 63 && j+m >= 0 && j+m <= 63 )
								myWalkability[i+k][j+m] = pathfinder->unwalkable;
				}		
			}
		}

		// add my own buildings (and opponents after that)
		const sint4 cid = game.get_client_player();
		sint4 oid = 0;
		if ( cid == 0 ) oid = 1;
		const Game::ObjCont &myobjs = game.get_objs(cid);

	  	FORALL (myobjs, it) {
			GameObj *gob = (*it)->get_GameObj();
			if (gob->bp_name() == "controlCenter" && *gob->sod.x1 > 0 && gob->sod.in_game)
			{
				halfBaseSize = (int)(*gob->sod.x2 - *gob->sod.x1) / 2;
				ScalarPoint topleft = getTileCoor(*gob->sod.x1,*gob->sod.y1);
				ScalarPoint botright = getTileCoor(*gob->sod.x2,*gob->sod.y2);
				for ( int i = topleft.x; i <= botright.x; i++ )
					for ( int j = topleft.y; j <= botright.y; j++ ) {
						// add buildings to pathfinder walkability
						pathfinder->walkability [i][j] = pathfinder->unwalkable;
						// add buildings to myWalkability
						myWalkability[i][j] = pathfinder->unwalkable;
					}
				if ( *gob->sod.x1 + halfBaseSize != myBaseCenter.x && *gob->sod.y1 + halfBaseSize != myBaseCenter.y )
				{
					ScalarPoint centerTile = getTileCoor(*gob->sod.x1 + halfBaseSize,*gob->sod.y1 + halfBaseSize);
					for ( int k = -4; k <= 4; k++ )
						for ( int m = -4; m <= 4; m++ )
							if ( centerTile.x+k >= 0 && centerTile.x+k <= 63 && centerTile.y+m >= 0 && centerTile.y+m <= 63 )
								myWalkability[centerTile.x+k][centerTile.y+m] = pathfinder->unwalkable;
				}
				else if ( *gob->sod.x1 + halfBaseSize == myBaseCenter.x && *gob->sod.y1 + halfBaseSize == myBaseCenter.y ){
					// reset border around myBase to walkable
					ScalarPoint baseTile = getTileCoor(myBaseCenter.x, myBaseCenter.y);
					for( int k = -3 ; k <= 3; k++){
						myWalkability[baseTile.x+k][baseTile.y-3] = pathfinder->walkable;
						myWalkability[baseTile.x-3][baseTile.y+k] = pathfinder->walkable;
						myWalkability[baseTile.x+k][baseTile.y+3] = pathfinder->walkable;
						myWalkability[baseTile.x+3][baseTile.y+k] = pathfinder->walkable;	
					}
				}
			}
		}

		//print mywalkable
		for ( int i = 0; i < 64; i++ ) {
			for ( int j = 0; j < 64; j++ )
			{
				if ( pathfinder->walkability[j][i] == pathfinder->unwalkable )
					cout << "X";
				else if ( myWalkability[j][i] == pathfinder->unwalkable )
					cout << "0";
				else if ( myWalkability[j][i] == pathfinder->walkable )
					cout << "_";
				else cout << " ";
			}
			cout << endl;
		}

		const Game::ObjCont &oppobjs = game.get_objs(oid);
		int counter = 0;
		closestOppBase = -1;
		int distance = 99999999;
		FORALL (oppobjs, it) {
	    		GameObj *gob = (*it)->get_GameObj();
			if (gob->bp_name() == "controlCenter" && *gob->sod.x1 > 0 && gob->sod.in_game)
			{					
//				cout << " opponent base @ " << *gob->sod.x1 + halfBaseSize << " , " << *gob->sod.y1 + halfBaseSize << endl;
				ScalarPoint topleft = getTileCoor(*gob->sod.x1,*gob->sod.y1);
				ScalarPoint botright = getTileCoor(*gob->sod.x2,*gob->sod.y2);
//				for ( int i = topleft.x; i <= botright.x; i++ )
//					for ( int j = topleft.y; j <= botright.y; j++ )
//						pathfinder->walkability [i][j] = pathfinder->unwalkable;
				oppBases[counter] = gob;			
				counter++;
			}
		}

		counter = 0;
		for ( int i = 0; i < 5; i++ ) {
			GameObj* gob = oppBases[i];
			int new_distance = (	(( *gob->sod.x1 + halfBaseSize ) - myBaseCenter.x)*
						(( *gob->sod.x1 + halfBaseSize ) - myBaseCenter.x) ) + 
					   (	(( *gob->sod.y1 + halfBaseSize ) - myBaseCenter.y)*
						(( *gob->sod.y1 + halfBaseSize ) - myBaseCenter.y) ) ;
			cout << "new distance = " << new_distance << endl;
			if ( new_distance < distance ) {
				closestOppBaseD.x = ( *gob->sod.x1 + halfBaseSize ) - myBaseCenter.x;
				closestOppBaseD.y = ( *gob->sod.y1 + halfBaseSize ) - myBaseCenter.y;
				distance = new_distance;
				closestOppBase = counter;
			}
			counter++;	
		}

		cout << "Nearest base for [" << myBaseCenter.x << "," << myBaseCenter.y << "] is: [" << *oppBases[closestOppBase]->sod.x1+halfBaseSize <<
		"," << *oppBases[closestOppBase]->sod.y1+halfBaseSize << "]" << endl;
//		cout << "with dx = " << closestOppBaseD.x << " and dy = " << closestOppBaseD.y << endl;
		// by now we know the ID of the closest oppBase (closestOppBase)
		// and its relative position (closestOppBaseD.x/.y = pixel points)
		// to our baseCenter (myBaseCenter.x/.y = pixel points)
		// We want to find a TILE which is walkable so we can put our units over there...
		// Note that from a base's center the nearest walkable tiles are at least 3 tiles away.
		int dx = closestOppBaseD.x;
		int dy = closestOppBaseD.y;
		horizontal = 0;
		vertical = 0;
		goal = getTileCoor(myBaseCenter.x, myBaseCenter.y);
		/*
			Sectors devided like:
			 \  7 | 8  /
			6 \   |   /  1
			--------------
			5 /   |   \  2
			 /  4 | 3  \
			
			and a rabbit looks like:
 		    / \     / \
                   {   }   {   }
                   {   {   }   }
                    \   \ /   /
                     \   Y   /
                     .-"`"`"-.
                   ,`         `.
                  /             \
                 /               \
                {     ;"";,       }
                {  /";`'`,;       }
                 \{  ;`,'`;.     /
                  {  }`""`  }   /}
                  {  }      {  // 
                  {||}      {  /
                  `"'       `"'
		*/

		int afstanden[tanks];
		if ( left )
		{
			goal.x -= 3;
			goal.y -= 2;
			left = 0;
			up = 1;
		} else {
			goal.x += 3;
			goal.y += 2;
			down = 1;
		}
		goal.x = goal.x*16+8;
		goal.y = goal.y*16+8;

		goAheadFormSquad= 1;

		for ( int i = 0; i < tanks; i++ ) {
			squad_tanks[i]->init();
		}

		firstTime = 0;
		for ( int i = 0; i < tanks; i++) {
			ScalarPoint poiu;
			poiu.x = -1;
			poiu.y = -1;
			prevPosition[i] = poiu;
		}
		prevPositionCnt = 0;

		// make the 'rangeerterrein' unwalkable
		ScalarPoint base = getTileCoor(myBaseCenter);
		for (int i = -3; i <= 3; i++ ) {
			for ( int j = -3; j <= 3; j++ ) {
				if ( i == 3 || i == -3 || j == 3 || j == -3 ) {
				pathfinder->walkability[base.x+i][base.y+j] = pathfinder->unwalkable;
				myWalkability[base.x+i][base.y+j] = pathfinder->unwalkable; }
			}
		}
		rangeerTerreinReleased = 0;
	}

	for (int i = 0; i < tanks; i++ ) {
		if ( (squad_tanks[i]->tank)->is_dead() ) {
			for ( int j = i+1; j < tanks; j++ ) {
				squad_tanks[j-1] = squad_tanks[j];
			}
			i--;
			tanks--;
		}
	}
	const Game::ObjCont &myobjs2 = game.get_objs(game.get_client_player());
	FORALL (myobjs2, it) {
		GameObj *gob = (*it)->get_GameObj();
		if ( gob->bp_name() == "controlCenter" && gob->is_dead() )
		{
			ScalarPoint topleft = getTileCoor(*gob->sod.x1,*gob->sod.y1);
			ScalarPoint botright = getTileCoor(*gob->sod.x2,*gob->sod.y2);
			for ( int i = topleft.x; i <= botright.x; i++ )
				for ( int j = topleft.y; j <= botright.y; j++ ) {
					// add buildings to pathfinder walkability
					pathfinder->walkability [i][j] = pathfinder->walkable;
					// add buildings to myWalkability
					myWalkability[i][j] = pathfinder->walkable;
				}
			
		}
	}

			GameObj *target = 0;
			GameObj* tempTarget2 = 0;
				// 5 tanks per squad
				for (int i = 0; i < 5; i++){
if (!(squad_tanks[i]->tank)->is_dead()){
					GameObj* tempTarget = 0;
					tempTarget = squad_tanks[i]->weakestInRange(game, 1 - game.get_client_player(), 1);
					// check whether this is the weakest oppTank of the squad
					if (tempTarget){
						// attack this target for now
//						cout << "tank[" << i << "] is attacking weakest tank in range" << endl;
	//					if (tempTarget->bp_name() == "tank" )
						squad_tanks[i]->attackWeakestInRange(game, tempTarget);
						if ( !target || *tempTarget->get_int_ptr("hp") < *target->get_int_ptr("hp")){
							// new weakest target
							target = tempTarget;				
//							cout << "tank[" << i << "] found new weakest squad target" << endl;
						}
					}
					else if (target && !squad_tanks[i]->isAttacking()){ // && target->bp_name() == "tank" 
	//					cout << "tank[" << i << "] has no target in range, moving to new weakest squad target" << endl;
						squad_tanks[i]->moveSelf(*target->sod.x, *target->sod.y);
						squad_tanks[i]->setGoal(*target->sod.x, *target->sod.y);
					}	
					// 5 tanks per squad and check for a controlcenter in range
					else {
						tempTarget2 = squad_tanks[i]->weakestInRange(game, 1 - game.get_client_player(), 1);
						if (tempTarget2  && *tempTarget2->sod.x >=0 && !squad_tanks[i]->isAttacking()){	
							squad_tanks[i]->moveSelf(*tempTarget2->sod.x, *tempTarget2->sod.y);
							squad_tanks[i]->setGoal(*tempTarget2->sod.x, *tempTarget2->sod.y);
//							cout << "moving to " << tempTarget2->bp_name() << "[" << *tempTarget2->sod.x1 << "," << *tempTarget2->sod.y1 << "] with health " << tempTarget2->get_const_int_ptr("hp") << " and x,y = " << *tempTarget2->sod.x << "," << *tempTarget2->sod.y <<  endl;
						}
						squad_tanks[i]->attackControlInRange(game, 1 - game.get_client_player());				
					}	
}
			}

if (!(target || tempTarget2)){
	if ( !squad_formed ) {
		squad_formed = true;
		for (int i = 0; i < tanks; i++ ) {
			if ( squad_tanks[i]->goalReached() == false ) squad_formed = false;
		}
		if ( !squad_formed ) cout<<"Squad Not formed!"<<endl;
		if ( !rangeerTerreinReleased ) {
			ScalarPoint base = getTileCoor(myBaseCenter);
			for ( int i = -3; i <= 3; i++ ) {
				pathfinder->walkability[base.x+i][base.y-3] = pathfinder->walkable;
				myWalkability[base.x+i][base.y-3] = pathfinder->walkable;
				pathfinder->walkability[base.x+i][base.y+3] = pathfinder->walkable;
				myWalkability[base.x+i][base.y+3] = pathfinder->walkable;
			}
			rangeerTerreinReleased = true;
		}
	}
	if ( !squad_formed && goAheadFormSquad ) {
//		setWalkability();

		//check whether a tank is stuck
		bool tankStuck = false;
		for ( int i = 0; i < tanks; i++) {
			ScalarPoint a = prevPosition[i];
			ScalarPoint b = currPosition[i];
			if ( a.x == b.x && a.y == b.y){
				cout << "tank cannot reach next goal position in formation" << endl;
				tankStuck = true;
			}
		}

	/*	if (prevPositionCnt > 10 && tankStuck) {
			//tank cannot reach next goal position in formation
			cout << "trying to position St. Elsewhere" << endl;
			prevPositionCnt = 0;	
		}
		else {*/
			//tank reached next goal position in formation
			prevPositionCnt++;
			for (int i = 0; i< tanks; i++ ) {
				prevPosition[i] = squad_tanks[i]->getPosition();
			}
			currentPosition.x = goal.x;
			currentPosition.y = goal.y;

			rotation = 0;
			if ( up ) rotation = 0;
			else if ( left ) rotation = 0.5*pi;
			else if ( down ) rotation = pi;
			else if ( right ) rotation = 1.5*pi;

			if ( up ) {
				ScalarPoint prePos;
				prePos.x = -1;
				prePos.y = -1;
				for ( int i = 0; i < tanks; i++ ){
					ScalarPoint newPoint = RotateFormation(*Column5[i], rotation);
					newPoint.x = goal.x + newPoint.x;
					newPoint.y = goal.y + newPoint.y;
					int posX = newPoint.x;
					int posY = newPoint.y;

					//save current tank position
					ScalarPoint poiu;
					poiu.x = newPoint.x;
					poiu.y = newPoint.y;
					currPosition[i] = poiu;

					squad_tanks[i]->setGoal(posX,posY);
					WalkabilityBackup[squad_tanks[i]->WalkableTile.x][squad_tanks[i]->WalkableTile.y] = pathfinder->walkable;
					WalkabilityBackup[squad_tanks[i]->UnwalkableTile.x][squad_tanks[i]->UnwalkableTile.y] = pathfinder->unwalkable;
				
					ScalarPoint tempGoal;
					ScalarPoint T = getTileCoor(squad_tanks[i]->getPosition());
					int speed = 1;
					if ( !squad_tanks[i]->goalReached() ) {
						ScalarPoint O = getTileCoor(goal);
						int dx = T.x - O.x;
						int dy = T.y - O.y;
						ScalarPoint tpGoal;
						if ( squad_tanks[i]->temporaryGoalReached() ) squad_tanks[i]->tempGoalSet = false;
						if ( squad_tanks[i]->tempGoalSet && !squad_tanks[i]->temporaryGoalReached() ) {
							tpGoal.x = squad_tanks[i]->temporaryGoal.x;
							tpGoal.y = squad_tanks[i]->temporaryGoal.y;
							squad_tanks[i]->moveSelfWOPathfinding(tpGoal.x, tpGoal.y, 1);
						} 
						else if ( dx == 5 && dy == -1 ) {
							if ( prePos.y > (goal.y-8) ) {
								tpGoal.x = goal.x + (6*16);
								tpGoal.y = goal.y - (1*16);
								squad_tanks[i]->temporaryGoal = tpGoal;
								squad_tanks[i]->tempGoalSet = true;
								squad_tanks[i]->moveSelfWOPathfinding(tpGoal.x, tpGoal.y, 1);
							}
						}
						else if ( dx == 6 && dy == 4 ) {
							if ( prePos.x < (goal.x+(5*16)+8) ) {
								tpGoal.x = goal.x + (6*16);
								tpGoal.y = goal.y + (5*16);
								squad_tanks[i]->tempGoalSet = true;
								squad_tanks[i]->temporaryGoal = tpGoal;
								squad_tanks[i]->moveSelfWOPathfinding(tpGoal.x, tpGoal.y, 1);
							}
						}
						else if ( dx == 1 && dy == 5 ) {
							if ( prePos.y < goal.y+(5*16)-8 ) {
								tpGoal.x = goal.x + (0*16);
								tpGoal.y = goal.y + (5*16);
								squad_tanks[i]->tempGoalSet = true;
								squad_tanks[i]->temporaryGoal = tpGoal;
								squad_tanks[i]->moveSelfWOPathfinding(tpGoal.x, tpGoal.y, 1);
							}					
						}
						else {
							if ( dx == 0 && dy > -1 ) {
								tpGoal.x = posX; tpGoal.y = posY;
							}
							else if ( dy == 5 && dx > 0 ) {
								tpGoal.x = goal.x + (1*16);
								tpGoal.y = goal.y + (5*16);
							}
							else if ( dx == 6 && dy < 5 ) {
								tpGoal.x = goal.x + (6*16);
								tpGoal.y = goal.y + (4*16);
							}
							else {
								tpGoal.x = goal.x + (5*16);
								tpGoal.y = goal.y - (1*16);
							}
							squad_tanks[i]->temporaryGoal = tpGoal;
							squad_tanks[i]->tempGoalSet = true;
							squad_tanks[i]->moveSelfWOPathfinding(tpGoal.x, tpGoal.y, 1);
						}
					}							
					prePos = squad_tanks[i]->getPosition();
				}
			} else if ( down ) {
				ScalarPoint prePos;
				prePos.x = -1;
				prePos.y = -1;
				for ( int i = 0; i < tanks; i++ ){
					ScalarPoint newPoint = RotateFormation(*Column5[i], rotation);
					newPoint.x = goal.x + newPoint.x;
					newPoint.y = goal.y + newPoint.y;
					int posX = newPoint.x;
					int posY = newPoint.y;

					//save current tank position
					ScalarPoint poiu;
					poiu.x = newPoint.x;
					poiu.y = newPoint.y;
					currPosition[i] = poiu;

					squad_tanks[i]->setGoal(posX,posY);
					WalkabilityBackup[squad_tanks[i]->WalkableTile.x][squad_tanks[i]->WalkableTile.y] = pathfinder->walkable;
					WalkabilityBackup[squad_tanks[i]->UnwalkableTile.x][squad_tanks[i]->UnwalkableTile.y] = pathfinder->unwalkable;
				
					ScalarPoint tempGoal;
					ScalarPoint T = getTileCoor(squad_tanks[i]->getPosition());
					int speed = 1;
					if ( !squad_tanks[i]->goalReached() ) {
						ScalarPoint O = getTileCoor(goal);
						int dx = T.x - O.x;
						int dy = T.y - O.y;
						ScalarPoint tpGoal;
						if ( squad_tanks[i]->temporaryGoalReached() ) squad_tanks[i]->tempGoalSet = false;
						if ( squad_tanks[i]->tempGoalSet && !squad_tanks[i]->temporaryGoalReached() ) {
							tpGoal.x = squad_tanks[i]->temporaryGoal.x;
							tpGoal.y = squad_tanks[i]->temporaryGoal.y;
							squad_tanks[i]->moveSelfWOPathfinding(tpGoal.x, tpGoal.y, 1);
						} 
						else if ( dx == -1 && dy == -5 ) {
							if ( prePos.y > (goal.y-(16*5)+8) ) {
								tpGoal.x = goal.x;
								tpGoal.y = goal.y - (5*16);
								squad_tanks[i]->temporaryGoal = tpGoal;
								squad_tanks[i]->tempGoalSet = true;
								squad_tanks[i]->moveSelfWOPathfinding(tpGoal.x, tpGoal.y, 1);
							}
						}
						else if ( dx == -1 && dy == 1 ) {
							if ( i+1 >= tanks ) { // sowieso bewegen
								tpGoal.x = goal.x;
								tpGoal.y = goal.y + (1*16);
								squad_tanks[i]->temporaryGoal = tpGoal;
								squad_tanks[i]->tempGoalSet = true;
								squad_tanks[i]->moveSelfWOPathfinding(tpGoal.x, tpGoal.y, 1);
							} else {
								ScalarPoint next = squad_tanks[i+1]->getPosition();
								if ( next.y < goal.y+8 ) { // bewegen als ie mag
									tpGoal.x = goal.x;
									tpGoal.y = goal.y + (1*16);
									squad_tanks[i]->temporaryGoal = tpGoal;
									squad_tanks[i]->tempGoalSet = true;
									squad_tanks[i]->moveSelfWOPathfinding(tpGoal.x, tpGoal.y, 1);
								}
							}
						}
						else {
							bool m = 0;
							if ( dx == 0 && dy <= 1 ) {
								tpGoal.x = posX; tpGoal.y = posY; m=1;
							}
							else if ( dx < 0 && dy == -5 ) {
								tpGoal.x = goal.x - (1*16);
								tpGoal.y = goal.y - (5*16); m=1;
							}
							else if ( dy == 1 && dx < 0 ) {
								tpGoal.x = goal.x - (1*16);
								tpGoal.y = goal.y + (1*16); m=1;								
							}
							else {  }
							if ( m ) {
								squad_tanks[i]->temporaryGoal = tpGoal;
								squad_tanks[i]->tempGoalSet = true;
								squad_tanks[i]->moveSelfWOPathfinding(tpGoal.x, tpGoal.y, 1);
							}
						}
					}							
					prePos = squad_tanks[i]->getPosition();
				}
			}
	}
	else {
		for ( int i = 0; i < tanks; i++ )
			if ( !(squad_tanks[i]->tank)->is_dead() )
				{}//			squad_tanks[i]->attackWeakestInRange(game, oid);
		if ( doOnce ) {
			cout << "Doing once" << endl;
			// reconfigure squad_tanks array
			
			//GAITank* temptank = squad_tanks[0];
			//squad_tanks[0] = squad_tanks[4];
			//squad_tanks[4] = squad_tanks[1];
			//squad_tanks[1] = squad_tanks[3];
			//squad_tanks[3] = temptank;
			
			// 2 blijft 2
			doOnce = 0;
			doTwice = 1;

			// Reorder to squad according to its nearest base [nb] (if nb.dy > 0 then reverse squad)

			if ( (up && closestOppBaseD.y > 0) || (down && closestOppBaseD.y < 0) ) {
				// reverse!
				GAITank* tempTank = squad_tanks[0];
				squad_tanks[0] = squad_tanks[4];
				squad_tanks[4] = tempTank;
				tempTank = squad_tanks[1];
				squad_tanks[1] = squad_tanks[3];
				squad_tanks[3] = tempTank;
				goal = squad_tanks[0]->getPosition();
			}
			
			beurt = 0;
		}
		if ( doTwice ) {
			// if tanks in range
		
//			else {
				//formFormation();
				for ( int i = 0; i < tanks; i++ ) {
					snakeHistory[i] = squad_tanks[i]->getPosition();
				}
				snakeWalk();
				beurt++;
				if ( beurt == 5 ) beurt = 0;
//			}
		}
	}
}
	return true;
}

bool GAISquad::inField(int x, int y) {
	if ( x < 0 ) return false;
	else if ( x > 63 ) return false;
	else if ( y < 0 ) return false;
	else if ( y > 63 ) return false;
	else return true;
}

double GAISquad::getAngle(ScalarPoint p1, ScalarPoint p2) {
	double dx = p2.x - p1.x;
	double dy = -(p2.y - p1.y);
	double angle;
	if ( dx > 0 )
		if ( dy > 0 )
			angle = atan(dy/dx);
		else	angle = 2*pi - atan(-dy/dx);
	else
		if ( dy > 0 )
			angle = pi - atan(dy/-dx);
		else 	angle = pi + atan(-dy/-dx);
	return angle - 1/2*pi;
}

ScalarPoint GAISquad::getTileCoor(int x, int y)
{
	ScalarPoint bla;
	bla.x = x/16;
	bla.y = y/16;
	return bla;
}

ScalarPoint GAISquad::getTileCoor(ScalarPoint x)
{
	x.x = x.x / 16;
	x.y = x.y / 16;
	return x;
}

int GAISquad::TileWalkable(const GameTile &tile)
{
	//const GameTile &tile = map(x,y);
	sint4 hNW, hNE, hSE, hSW;
	Tile::Split split;
	// get corner heights and split type
	tile.get_topo(hNW, hNE, hSE, hSW, split);
	if (hNW > 0 || hNE > 0 || hSE > 0 || hSW > 0 || split > 0)
		return pathfinder->unwalkable;
	else 
		return pathfinder->walkable;
}

ScalarPoint GAISquad::RotateFormation(ScalarPoint p, double angle) {
/*	double afstand = sqrt(p.x*p.x + p.y*p.y);
	ScalarPoint newpoint;
	newpoint.x = afstand * cos(angle);
	newpoint.y = afstand * sin(angle);
	return newpoint; */

	double afstand = sqrt(p.x*p.x + p.y*p.y);
	if ( afstand == 0 ) { return p; }
	else {
		ScalarPoint newpoint;
		newpoint.x = -(afstand * cos(angle+acos(p.x/afstand)));
		newpoint.y = afstand * sin(angle+acos(p.x/afstand));
		return newpoint;
	}
}

void GAISquad::setWalkability(){
	
	for ( int i = 0; i < 64; i++ )
		for ( int j = 0; j < 64; j++ )
		{
			WalkabilityBackup[i][j] = pathfinder->walkability[i][j];
			pathfinder->setTile(myWalkability[i][j],i,j);
		}
}

void GAISquad::resetWalkability(){
	for ( int i = 0; i < 64; i++ )
		for ( int j = 0; j < 64; j++ )
			pathfinder->setTile(WalkabilityBackup[i][j],i,j);
}

void GAISquad::setWalkabilityBackupTile(int x, int y, int value) {
	WalkabilityBackup[x][y] = value;
}

void GAISquad::formFormation()
{
	cout << "Doing twice" << endl;

	for ( int i = 0; i < tanks; i++ ) {				
		ScalarPoint newPoint = RotateFormation(*Line5[i], rotation);
			newPoint.x = goal.x + newPoint.x;
			newPoint.y = goal.y + newPoint.y;
			int posX = newPoint.x;
			int posY = newPoint.y;
			//squad_tanks[i]->moveSelf(posX,posY);
			//squad_tanks[i]->setGoal(posX,posY);
			squad_tanks[i]->move(posX,posY);
			WalkabilityBackup[squad_tanks[i]->WalkableTile.x][squad_tanks[i]->WalkableTile.y] = pathfinder->walkable;
			WalkabilityBackup[squad_tanks[i]->UnwalkableTile.x][squad_tanks[i]->UnwalkableTile.y] = pathfinder->unwalkable;
			// cout << "posX = " << posX << " , posY = " << posY << endl;
	}

}

void GAISquad::snakeWalk()
{
	int distance = 9999999;
	int counter = 0;
	for ( int i = 0; i < 5; i++ ) {
		GameObj* gob = oppBases[i];
		if ( gob->sod.x1 >= 0 && gob->sod.in_game && !gob->is_dead() ) {
		int new_distance = (	(( *gob->sod.x1 + halfBaseSize ) - goal.x)*
					(( *gob->sod.x1 + halfBaseSize ) - goal.x) ) + 
				   (	(( *gob->sod.y1 + halfBaseSize ) - goal.y)*
					(( *gob->sod.y1 + halfBaseSize ) - goal.y) ) ;
		if ( new_distance < distance ) {
			closestOppBaseD.x = ( *gob->sod.x1 + halfBaseSize ) - goal.x;
			closestOppBaseD.y = ( *gob->sod.y1 + halfBaseSize ) - goal.y;
			distance = new_distance;
			closestOppBase = counter;
		}}
		counter++;	
	}
	int posX = *oppBases[closestOppBase]->sod.x1+halfBaseSize;
	int posY = *oppBases[closestOppBase]->sod.y1+halfBaseSize;
	snakeHistory[1] = squad_tanks[0]->getPosition();
	goal = squad_tanks[0]->getPosition();
	squad_tanks[0]->moveSelf(posX,posY);
	for ( int i = 1; i < tanks; i++ ) {
		if ( i+1 < tanks ) { snakeHistory[i+1] = squad_tanks[i]->getPosition(); }
		squad_tanks[i]->moveSelf(snakeHistory[i].x, snakeHistory[i].y);
	}
}
