// $Id: CircleEventHandler.C,v 1.6 2007/05/26 07:01:08 pjungkun Exp $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

// A sample client for ORTS that shows how to access client data

// todo/questions:
//
// game scripts:
// - what to do with workers during build time
// - similarly, how to define mining?
// - how is attack range defined (distance between centers/hulls)?
//
// client:
// - how to access global player data?
// - how to compute the set of visible tiles?
// - how to compute tiles visible from a unit?
// - how to decide what can be attacked by a specific unit?

// FORS/FORALL/ERR macros are defined in orts3/libs/kernel/src/Global.H

#include "CircleEventHandler.H"
#include "formationCircle.H"
#include "GameStateModule.H"
#include "GfxModule.H"
#include "Game.H"
#include "GameChanges.H"
#include "GameTile.H"
#include "ServerObjData.H"
#include "Options.H"
#include "GUI.H"

#include "DrawOnTerrain2D.H"

using namespace std;
formationCircle handlerCircle;

// tile types from core.bp
// fixme: game script reflection would help here
// in the competition we won't use water tiles
// TILE_CLIFF: non-traversable by ground units

enum { TILE_UNKNOWN=0, TILE_WATER=1, TILE_GROUND=2, TILE_CLIFF=3 };

bool CircleEventHandler::handle_event(const Event& e)
{
  if (e.get_who() == GameStateModule::FROM) {
    
    //cout << "Event: " << e.who() << " " << e.what() << endl;
    
    if (e.get_what() == GameStateModule::STOP_MSG) {
      state.quit = true; return true;
    }
    
    if (e.get_what() == GameStateModule::READ_ERROR_MSG) {
      state.quit = state.error = true; return true;
    }
    
    if (e.get_what() == GameStateModule::VIEW_MSG) {
    	const sint4 me = state.gsm->get_game().get_client_player();
    	sint4 npc = state.gsm->get_game().get_player_num();
    	
    	FORALL (state.gsm->get_changes().new_objs, i) {
        GameObj *gob = (*i)->get_GameObj();
        if (gob == 0) continue;
        if (*gob->sod.owner == me) {
    			if(gob->bp_name() == "marine") {
							marines.push_back(gob);
							handlerCircle.insertShooter(gob);
					}
				}
				else if (*gob->sod.owner == npc) continue;
				else {
					if(gob->bp_name() == "marine") {
							handlerCircle.addTarget(gob);
							//adding the opponents and their healthpoints into the opponents map
							opponents.insert(make_pair(gob, (gob->get_val("hp")).to_int()));
					}
				}
			}
			FORALL (state.gsm->get_changes().changed_objs, i) {
    		GameObj *gob = (*i)->get_GameObj();
    		 if (gob == 0) continue;
    		 if (*gob->sod.owner != me && *gob->sod.owner != npc) {
    		 	if(gob->bp_name() == "marine") {
						if ((gob->get_val("hp")).to_int() != opponents[gob]) {
							opponents[gob] = (gob->get_val("hp")).to_int();
							//shot = true;
						}   		 	
    		 	}
    		 }
    	}
    	
    	FORALL (state.gsm->get_changes().dead_objs, i) {
    		GameObj *gob = (*i)->get_GameObj();
    		if (gob == 0) continue;
    		if (*gob->sod.owner == me) {
    			handlerCircle.removeShooter(gob);
    		}
    	}
 
      compute_actions();
      state.gsm->send_actions();

      Game &game = state.gsm->get_game();	

      // gfx and frame statistics
      
      if (state.gfxm) state.gfxm->process_changes();
      
      sint4 vf = game.get_view_frame();
      sint4 af = game.get_action_frame();
      sint4 sa = game.get_skipped_actions();
      sint4 ma = game.get_merged_actions();

      if ((vf % 20) == 0) cout << "[frame " << vf << "]" << endl;
	
      if (vf != af || sa || ma > 1) {
            
	cout << "frame " << vf;
	if (af < 0) cout << " [no action]";
	else if (vf != af) cout << " [behind by " << (vf - af) << " frame(s)]";
	if (sa) cout << " [skipped " << sa << "]";
	if (ma > 1) cout << " [merged " << ma << "]";
	cout << endl;
      }

      if (state.gfxm) {
	
	// don't draw if we are far behind
	if (abs(vf-af) < 10) {
	  state.gfxm->draw();
	  state.just_drew = true;
	}
      }

      if (state.gui) {
        state.gui->event();
        //        state.gui->display();
        if (state.gui->quit) exit(0);
      }

      // as a hack for flickering, gui->display() is called in dot.start()
      // so call everything that draws to the debug window between start() and end()
      DrawOnTerrain2D dot(state.gui);
      dot.start();
      dot.draw_line(-60, 10, -20, 10, Vec3<real4>(1.0, 1.0, 1.0));
      dot.draw_line(-60, 10, -65, 15, Vec3<real4>(1.0, 1.0, 1.0));
      dot.draw_line(-44, 10, -56, 40, Vec3<real4>(1.0, 1.0, 1.0));
      dot.draw_line(-28, 10, -36, 40, Vec3<real4>(1.0, 1.0, 1.0));
      dot.draw_line(-36, 40, -30, 34, Vec3<real4>(1.0, 1.0, 1.0));

      dot.draw_circle(-40, 25, 35, Vec3<real4>(1.0, 0.0, 1.0));

      dot.end();

      return true;
    }
  } 
  return false;
}


void CircleEventHandler::compute_actions()
{
  //cout << endl;

  //cout << "compute actions" << endl;

  const Game &game = state.gsm->get_game();
  const sint4 cid = game.get_client_player();
  const Map<GameTile> &map = game.get_map();
  //const Game::ObjCont &objs = game.get_objs(cid);
  const sint4 points_x = map.get_width()  * game.get_tile_points();
  const sint4 points_y = map.get_height() * game.get_tile_points();  
  
  
  
  if ( ! state.isInitialized )
	{
		std::vector<GameObj*> assMarines;
		
		state.isInitialized = true;
		if (cid==0)
    {
			state.leftToRight = true;
			handlerCircle.setCenter(ScalarPoint(points_x/8, points_y/2));
			// loop through circle slots
			for (int slot=49; slot>=0; --slot) {
			// for each slot 
			//			loop through marines and assign closest one to this slot
				vector<GameObj*>::iterator marine;
				vector<GameObj*>::iterator assM;
				GameObj* closest = 0;
				double dist;
				double min = 32000;
				for (marine = marines.begin(); marine != marines.end(); ++marine) {
					GameObj *gob = (*marine)->get_GameObj();
					dist = sqrt ( (*gob->sod.x - handlerCircle.getSlotPos(slot).x) * (*gob->sod.x - handlerCircle.getSlotPos(slot).x) +
										  (*gob->sod.y - handlerCircle.getSlotPos(slot).y) * (*gob->sod.y - handlerCircle.getSlotPos(slot).y)
					 				 );
					//cout << dist << endl;
				
					if (dist < min) {
						min = dist;
						closest = gob;
						assM = marine;	
					}			
				  
				}
				handlerCircle.setSlot(closest, slot);
				assMarines.push_back(*assM);
				marines.erase(assM);
		  	//cout << closest->get_obj_id(closest) << endl;		 
				//	cout << "slot " << slot << "pos (" << handlerCircle.getSlotPos(slot).x << "," << handlerCircle.getSlotPos(slot).y << ")" << endl;
			}
		}
		else
		{
			state.leftToRight = false;
			handlerCircle.setCenter(ScalarPoint(7*points_x/8, points_y/2));
			// loop through circle slots
			for (int slot=0; slot<50; ++slot) {
			// for each slot 
			//			loop through marines and assign closest one to this slot
				vector<GameObj*>::iterator marine;
				vector<GameObj*>::iterator assM;
				GameObj* closest = 0;
				double dist;
				double min = 32000;
				for (marine = marines.begin(); marine != marines.end(); ++marine) {
					GameObj *gob = (*marine)->get_GameObj();
					dist = sqrt ( (*gob->sod.x - handlerCircle.getSlotPos(slot).x) * (*gob->sod.x - handlerCircle.getSlotPos(slot).x) +
										  (*gob->sod.y - handlerCircle.getSlotPos(slot).y) * (*gob->sod.y - handlerCircle.getSlotPos(slot).y)
					 				 );
					//cout << dist << endl;
				
					if (dist < min) {
						min = dist;
						closest = gob;
						assM = marine;	
					}			
				  
				}
				handlerCircle.setSlot(closest, slot);
				assMarines.push_back(*assM);
				marines.erase(assM);
		  	//cout << closest->get_obj_id(closest) << endl;		 
				//	cout << "slot " << slot << "pos (" << handlerCircle.getSlotPos(slot).x << "," << handlerCircle.getSlotPos(slot).y << ")" << endl;
			}
		}
		
		cout << "center = (" << handlerCircle.getCenter().x << "," << handlerCircle.getCenter().y << ")" << endl;
		
		
		marines = assMarines;
		
		/* some debug code
		cout << "===========================================================" << endl;
		vector<GameObj*>::iterator marine;
		for (marine = marines.begin(); marine != marines.end(); ++marine)
		{
			cout << (*marine)->get_obj_id(*marine) << endl;
		}
				cout << "===========================================================" << endl;
		*/
	
	}
	

  handlerCircle.updateTargetList(state.gsm);
 
  handlerCircle.shoot(state.gsm);
  
  handlerCircle.moveCircle(state.gsm);
	


  // - global player/game variables

  // ... to be implemented
}
