// $Id$

#include "Game1EventHandler.h"
#include "GameStateModule.H"
#include "GfxModule.H"
#include "Game.H"
#include "GameChanges.H"
#include "GameTile.H"
#include "ServerObjData.H"
#include "Options.H"
#include "GUI.H"

#include "SimpleTerrain.H"
#include "TerrainModule.H"
#include "Distance.H"

#include "LocalWorker.h"
#include <assert.h>


using namespace std;


enum { TILE_UNKNOWN=0, TILE_WATER=1, TILE_GROUND=2, TILE_CLIFF=3 };

Game1EventHandler::Game1EventHandler(SampleAIState &state_, sint4 seed) : state(state_) { 

  FROM = EventFactory::new_who();
  minerals.clear();
  minerals.clear();
  controlCenter = NULL;
  rand.srand(seed);
  numPF=0;
}


bool Game1EventHandler::handle_event(const Event& e){

  if (e.get_who() == GameStateModule::FROM) {
    if (e.get_what() == GameStateModule::STOP_MSG) {
      state.quit = true; return true;
    }
    if (e.get_what() == GameStateModule::READ_ERROR_MSG) {
      state.quit = state.error = true; return true;
    }
    if (e.get_what() == GameStateModule::VIEW_MSG) {
      compute_actions_state_vew();
      state.gsm->send_actions();
      return true;
    }
  }
  else if (e.get_who() == TerrainModule::FROM){

    const TerrainModule::PathStatusEvent* pse = 
      dynamic_cast<const TerrainModule::PathStatusEvent*>(&e);

    if( pse ) {
      GameObj* gob = pse->get_obj();
      //cout << "=========  cuurent object";

      sint4 type = pse->get_what();
      if (type == TerrainModule::PATH_SUCCESS){
        //cout << "TerrainModule::PATH_SUCCESS" << endl;
        //printGemeObjectInfo(gob);
        compute_actions_path_success(gob);
      }
      else if (type == TerrainModule::PATH_FAILURE) {
        //cout << "TerrainModule::PATH_FAILURE" << endl;
        sint4 why = pse->get_details();
        if (why == TerrainModule::PATH_PLAN_FAILURE){
          // cout << "TerrainModule::PATH_PLAN_FAILURE" << endl;
          //printGemeObjectInfo(gob);
          compute_actions_path_plan_failure(gob);
        }
        else if (why == TerrainModule::PATH_MOVE_FAILURE){
          //cout << "TerrainModule::PATH_MOVE_FAILURE" << endl;
          //printGemeObjectInfo(gob);
          compute_actions_path_move_failure(gob);            
        }
        else {
          //cout << "Path failure for unknown reason" << endl;
          assert(0); // Should never get here
        }
      }
      else {
        //cout << "Message from TerrainModule of unknown type" << endl;
        assert(0); // Should never get here
      }
    }
    return true;
  }
  return false;
}


void Game1EventHandler::compute_actions_state_vew() {

  //cout << "compute actions" << endl;
  const GameChanges &changes = state.gsm->get_changes();

  // sets init positions of minerals and command center
  for(typeof(changes.new_objs.begin()) it = changes.new_objs.begin(); it != changes.new_objs.end(); it++){
    GameObj* gob=(*it)->get_GameObj();
    if(gob->bp_name() == "worker"){
      workers.push_back(LocalWorker(gob));
    }
    else if(gob->bp_name() == "mineral"){
      minerals.insert(gob);
    }
    else if(gob->bp_name() == "sheep"){
      sheeps.insert(gob);
    }
    else if(gob->bp_name() == "controlCenter"){
      controlCenter=gob;
    }
  }

  // if a mineral is empty
  for(typeof(changes.vanished_objs.begin()) it = changes.vanished_objs.begin(); it != changes.vanished_objs.end(); it++){
    GameObj* gob=(*it)->get_GameObj();
    if(gob->bp_name() == "mineral"){
      minerals.erase(gob);
    }
  }

  // process only 1 worker
  //process(&(*workers.begin()));
  
  // process all 
  //cout << "<<<<<<<<<<<<<<<<<< New iteration" << endl;     
  for (std::vector<LocalWorker>::iterator it = workers.begin(); it != workers.end(); it++){
    process(&(*it));
  }
  //cout << endl;     

}

void Game1EventHandler::compute_actions_path_success(GameObj* gob) {

  LocalWorker* lw = NULL; 
 
  lw = findWorker(gob);
  if (lw!=NULL){
    switch (lw->getState()){
      case WALKING_TO_MINERAL_PF:
        numPF--;
        doWalkToMineralFinish(lw);
        lw->setState(WALKING_TO_MINERAL_FINISHING);
        break;
      case WALKING_TO_CONTROL_CENTER_PF:
        numPF--;
        doWalkToControlCenterFinish(lw);
        lw->setState(WALKING_TO_CONTROL_CENTER_FINISHING);
        break;
      case IDLE:
      case WALKING_TO_MINERAL:
      case WALKING_TO_MINERAL_STUCK:
      case WALKING_TO_MINERAL_FINISHING:
      case MINING:
      case WALKING_TO_CONTROL_CENTER:
      case WALKING_TO_CONTROL_CENTER_STUCK:
      case WALKING_TO_CONTROL_CENTER_FINISHING:
      case RETURNING_RESOURCES:
	break;		
    }
  }
}

void Game1EventHandler::compute_actions_path_plan_failure(GameObj* gob){

  LocalWorker* lw = NULL; 

  lw = findWorker(gob);
  if (lw!=NULL){
    switch (lw->getState()){
      case WALKING_TO_MINERAL_PF:
        numPF--;
        lw->setState(IDLE);
        break;
      case WALKING_TO_CONTROL_CENTER_PF:
        numPF--;
        doWalkToRandomPosition(lw);
        lw->setState(WALKING_TO_CONTROL_CENTER_FINISHING);
        break;
      case IDLE:
      case WALKING_TO_MINERAL:
      case WALKING_TO_MINERAL_STUCK:
      case WALKING_TO_MINERAL_FINISHING:
      case MINING:
      case WALKING_TO_CONTROL_CENTER:
      case WALKING_TO_CONTROL_CENTER_STUCK:
      case WALKING_TO_CONTROL_CENTER_FINISHING:
      case RETURNING_RESOURCES:
	break;		
      }
    }
}

void Game1EventHandler::compute_actions_path_move_failure(GameObj* gob){
 
  LocalWorker* lw = NULL; 

  lw = findWorker(gob);
  if (lw!=NULL){
    switch (lw->getState()){
      case WALKING_TO_MINERAL_PF:
        numPF--;
        doWalkToRandomPosition(lw);
        lw->setState(WALKING_TO_MINERAL_FINISHING);
        break;
      case WALKING_TO_CONTROL_CENTER_PF:
        numPF--;
        doWalkToRandomPosition(lw);
        lw->setState(WALKING_TO_CONTROL_CENTER_FINISHING);
        break;
      case IDLE:
      case WALKING_TO_MINERAL:
      case WALKING_TO_MINERAL_STUCK:
      case WALKING_TO_MINERAL_FINISHING:
      case MINING:
      case WALKING_TO_CONTROL_CENTER:
      case WALKING_TO_CONTROL_CENTER_STUCK:
      case WALKING_TO_CONTROL_CENTER_FINISHING:
      case RETURNING_RESOURCES:
	break;		
    }
  }
}


void Game1EventHandler::printGemeObjectInfo(GameObj *gob){

  const ServerObjData &sod = gob->sod;

  cout << "bp: " << gob->bp_name();

  if (!sod.in_game) { 
    cout << " no gameobj" << endl;
    return;
  }

  if (*sod.x < 0) {
    cout << "not on playfield" << endl;
    return;
  }

  cout << " zcat: " << *sod.zcat
    << " max-speed: " << *sod.max_speed
    << " speed: " << *sod.speed 
    << " moving: " << *sod.is_moving 
    << " sight: " << *sod.sight
    << " X : " << *sod.x
    << " Y : " << *sod.y;

  if (*sod.shape == Object::CIRCLE) {
    cout << " CIRCLE loc: (" << *sod.x << "," << *sod.y << ")"
      << " r: "<< *sod.radius;

  } else if (*sod.shape == Object::RECTANGLE) {
    cout << " RECT dim: (" << *sod.x1 << "," << *sod.y1 << ")-("
      << *sod.x2 << "," << *sod.y2 << ")";
  } else {
    cout << " UNKNOWN SHAPE";
  }

  // - other object properties
  // heading: 0 = stopped, 1...GAME_CONST::HEADING_N
  const sintptr *hd = gob->get_int_ptr("heading");
  if (hd){ 
    cout << " hd: " << GameConst::angle_from_dir(*hd, GameConst::HEADING_N);
  }

  cout << endl;

}

LocalWorker* Game1EventHandler::findWorker(GameObj *gob){

  LocalWorker* result = NULL;
  for (std::vector<LocalWorker>::iterator it = workers.begin(); it != workers.end(); it++){
    if ( (*it).getGOB() == gob) {
      result = &(*it);
      break;
    }
  }
  assert(result);
  return result;
}


void Game1EventHandler::moveWorker(LocalWorker* lw, sint4 x, sint4 y){

  TerrainBase::Loc location(x, y);

  call_handlers( TerrainModule::PathEvent(FROM, TerrainModule::FIND_PATH_MSG,
    TerrainBase::Obj2LocTask( (Object*)(lw->getGOB()), location)));
}


void Game1EventHandler::process(LocalWorker* lw){

  //const Game &game = state.gsm->get_game();
  Vector<sint4> params;
  bool nearToMineral = false;
  
  switch (lw->getState()){
    case IDLE:
      if ((*(lw->getGOB()->sod.speed) == 0)&&!(lw->getGOB()->is_pending_action())){
        doAssignMineral(lw);
        doWalkToMineralFinish(lw);
        lw->setState(WALKING_TO_MINERAL_FINISHING);
        //cout << "----------I am walking to mineral " << endl;
      }
      break;
    case WALKING_TO_MINERAL_FINISHING:
      if ((*(lw->getGOB()->sod.speed) == 0)&&!(lw->getGOB()->is_pending_action())){
        nearToMineral = false;
        for (std::set<GameObj*>::iterator it = minerals.begin(); it != minerals.end(); it++){
          if (Distance::distance(*(lw->getGOB()), *(*it)) < 3.0){
            nearToMineral = true;
            lw->setTargetMineral(*it);
            doMine(lw);
                  lw->setAttempt(0);
            lw->setState(MINING);
            //cout << "----------I am mining " << endl;
            break;
          }
        }
        if (!nearToMineral){
          doWalkToRandomPosition(lw);
          lw->setState(WALKING_TO_MINERAL_STUCK);
          //cout << "----------I am stuck walking to mineral " << endl;
        }
      }
      break;
    case WALKING_TO_MINERAL_STUCK:
      if ((*(lw->getGOB()->sod.speed) == 0)&&!(lw->getGOB()->is_pending_action())){
        if (lw->getAttempt() > 5 ){
          if (numPF<20){
            numPF++;
            lw->setAttempt(0);
            doAssignMineralAproach(lw);
            doWalkToMineralPF(lw);
            lw->setState(WALKING_TO_MINERAL_PF);
            //cout << "----------I will try PF " << endl;
          }
          else{
            lw->setState(IDLE);
            //cout << "----------I will try new mineral " << endl;
          }
        }else{
          doWalkToMineralFinish(lw);
          lw->setState(WALKING_TO_MINERAL_FINISHING);
          //cout << "----------I am walking to mineral again " << endl;
        }
      }
      break;
    case MINING:
      if(lw->getGOB()->component("pickaxe")->get_int("active") == 0){
        if(lw->getGOB()->get_int("minerals") < 10){
          //cout << "----------not enough minerals " << endl;
          lw->setState(IDLE);
        }
        else{
          doWalkToControlCenterFinish(lw);
          lw->setState(WALKING_TO_CONTROL_CENTER_FINISHING);
          //cout << "----------I am walking to control center " << endl;
        }
      }
      break;
    case WALKING_TO_CONTROL_CENTER_FINISHING:
      if ((*(lw->getGOB()->sod.speed) == 0)&&!(lw->getGOB()->is_pending_action())){
        if(Distance::distance(*(lw->getGOB()), *controlCenter) < 3.0){
          doRerurnRecources(lw);
          lw->setState(RETURNING_RESOURCES);
          //cout << "----------I am returning minerals " << endl;
        }
        else{
          doWalkToRandomPosition(lw);
          lw->setState(WALKING_TO_CONTROL_CENTER_STUCK);
          //cout << "----------I am stuck walking to control center " << endl;
        } 
      }
      break;
    case WALKING_TO_CONTROL_CENTER_STUCK:
      if ((*(lw->getGOB()->sod.speed) == 0)&&!(lw->getGOB()->is_pending_action())){
        if (lw->getAttempt() > 5 ){
          if (numPF<20){
            numPF++;
            lw->setAttempt(0);
            doAssignControlCenterAproach(lw);
            doWalkToControlCenterPF(lw);
            lw->setState(WALKING_TO_CONTROL_CENTER_PF);
            //cout << "----------I will try PF for control center" << endl;
          }
          else{
            doWalkToControlCenterFinish(lw);
            lw->setState(WALKING_TO_CONTROL_CENTER_FINISHING);
            //cout << "----------I am walking to control center again " << endl;
          }
        }else{
          doWalkToControlCenterFinish(lw);
          lw->setState(WALKING_TO_CONTROL_CENTER_FINISHING);
          //cout << "----------I am walking to control center again " << endl;
        }
      }
      break;
    case RETURNING_RESOURCES:
      if(lw->getGOB()->get_int("minerals") == 0 ){
        lw->setAttempt(0);
        doWalkToMineralFinish(lw);
        lw->setState(WALKING_TO_MINERAL_FINISHING);
        //cout << "----------I am IDLE from returning minerals" << endl;
      }else{
        //cout << "----------I am still returning minerals" << endl;
      }
      break;
    default: 
      break;
  }
}

void Game1EventHandler::doAssignMineral(LocalWorker* lw){

  int i;
  int r;
  set<GameObj*>::size_type b;  
  set<GameObj*>::iterator it;

  b = minerals.size();
  r = std::rand() % static_cast<int>(b);
  for (i = 0, it = minerals.begin(); i < r ; i++, it++){}
  lw->setTargetMineral(*it);
  lw->setAttempt(0);
  lw->setTX(*(lw->getTargetMineral())->sod.x);
  lw->setTY(*(lw->getTargetMineral())->sod.y);

}

void Game1EventHandler::doAssignMineralAproach(LocalWorker* lw){

  real8 minDistance = 0; 
  real8 tmpDistance = 0; 
  sint4 target_x = 0;
  sint4 target_y = 0;
  sint4 temp_x = 0;
  sint4 temp_y = 0;

  if (lw->getTargetMineral() == NULL){
    doAssignMineral(lw);
  }

  minDistance = Distance::distance(*(lw->getGOB()), *(lw->getTargetMineral())->sod.x, *(lw->getTargetMineral())->sod.y);
  for (int i = 0; i < 8; i++){
    temp_x = *(lw->getTargetMineral())->sod.x + static_cast<int> (sin(i*3.1415/4.0)*4.0);
    temp_y = *(lw->getTargetMineral())->sod.y + static_cast<int> (cos(i*3.1415/4.0)*4.0);
    tmpDistance = Distance::distance(*(lw->getGOB()), temp_x, temp_y);
    if (tmpDistance < minDistance){
      minDistance = tmpDistance;
      target_x = temp_x;
      target_y = temp_y;
    }
  }

  lw->setTX(target_x);
  lw->setTY(target_y);

}

void Game1EventHandler::doWalkToMineralPF(LocalWorker* lw){

  moveWorker(lw, lw->getTX(), lw->getTY());
}

void Game1EventHandler::doWalkToMineralFinish(LocalWorker* lw){

  sint4 target_x = *(lw->getTargetMineral())->sod.x;
  sint4 target_y = *(lw->getTargetMineral())->sod.y;

  lw->setTX(target_x);
  lw->setTY(target_y);

  lw->moveToXY();
}

void Game1EventHandler::doMine(LocalWorker* lw){
    
  const Game &game1 = state.gsm->get_game();
  Vector<sint4> params;
 
  params.clear();
  params.push_back(game1.get_cplayer_info().get_id(lw->getTargetMineral()));
  lw->getGOB()->component("pickaxe")->set_action("mine", params);
}

void Game1EventHandler::doAssignControlCenterAproach(LocalWorker* lw){

  real8 minDistance = 0; 
  real8 tmpDistance = 0; 
  sint4 target_x = 0;
  sint4 target_y = 0;
  sint4 temp_x = 0;
  sint4 temp_y = 0;

  minDistance = Distance::distance(*(lw->getGOB()), *controlCenter->sod.x, *controlCenter->sod.y);
  for (int i = 0; i < 8; i++){
    temp_x = *controlCenter->sod.x + static_cast<int> (sin(i*3.1415/2.0)*40.0);
    temp_y = *controlCenter->sod.y + static_cast<int> (cos(i*3.1415/2.0)*40.0);
    tmpDistance = Distance::distance(*(lw->getGOB()), temp_x, temp_y);
    if (tmpDistance < minDistance){
      minDistance = tmpDistance;
      target_x = temp_x;
      target_y = temp_y;
    }
  }

  lw->setTX(target_x);
  lw->setTY(target_y);

 // sint4 target_x = *controlCenter->sod.x2 + 3.00;
 // sint4 target_y = *controlCenter->sod.y2 + 3.00;
}

void Game1EventHandler::doWalkToControlCenterPF(LocalWorker* lw){

  moveWorker(lw, lw->getTX(), lw->getTY());
}


void Game1EventHandler::doWalkToControlCenterFinish(LocalWorker* lw){

  sint4 target_x = *controlCenter->sod.x;
  sint4 target_y = *controlCenter->sod.y;

  lw->setTX(target_x);
  lw->setTY(target_y);

  lw->moveToXY();
}

void Game1EventHandler::doRerurnRecources(LocalWorker* lw){

  const Game &game2 = state.gsm->get_game();
  Vector<sint4> params;

  params.clear();
  params.push_back(game2.get_cplayer_info().get_id(controlCenter));
  lw->getGOB()->set_action("return_resources", params);
  lw->setState(RETURNING_RESOURCES);
  //cout << "----------return resources " << endl;

}

void Game1EventHandler::doWalkToRandomPosition(LocalWorker* lw){

  int i = lw->getAttempt();
  i++;
  sint4 target_x = *(lw->getGOB()->sod.x) - 20 + std::rand() % 40;
  sint4 target_y = *(lw->getGOB()->sod.y) - 20 + std::rand() % 40;


  lw->setAttempt(i);
  lw->setTX(target_x);
  lw->setTY(target_y);

  lw->moveToXY();
}

