// $Id$

#ifndef GAME1_EVENT_HANDLER_H
#define GAME1_EVENT_HANDLER_H

#include "EventHandler.H"
#include "Random.H"
#include "ClientModule.H"

#include "LocalWorker.h"

class GameStateModule;
class GfxModule;
class GUI;

// This class gives the event handler access to all game and gfx data

struct SampleAIState {

  GameStateModule *gsm;
  GfxModule       *gfxm; // !=0 : gfx used
  GUI             *gui;  // !=0 : debug gfx used
  bool just_drew;  // just drew a gfx frame
  bool quit;       // true => quit game
  bool error;      // true <=> error occurred
  
  SampleAIState(GameStateModule *gsm_, GfxModule *gfxm_, GUI *gui_) :
    gsm(gsm_), gfxm(gfxm_), gui(gui_)
  {
    just_drew = quit = error = false;
  }

};

//  Handles server events
class Game1EventHandler : public EventHandler, public ClientModule {

public: 
  sint4 FROM;
  SampleAIState &state;
  Game1EventHandler(SampleAIState &state, sint4 seed); 
  bool handle_event(const Event &e);

private: 
  std::vector<LocalWorker> workers;
  std::set<GameObj*> minerals;
  std::set<GameObj*> sheeps;
  GameObj* controlCenter;
  int totalMinerals;
  int numPF;

  void compute_actions_state_vew();
  void compute_actions_path_success(GameObj* gob);
  void compute_actions_path_plan_failure(GameObj* gob);
  void compute_actions_path_move_failure(GameObj* gob);

  void printGemeObjectInfo(GameObj *gob);
  LocalWorker* findWorker(GameObj *gob);
  void process(LocalWorker* lw);
  
  void doAssignMineral(LocalWorker* lw);
  void doAssignMineralAproach(LocalWorker* lw);
  void doWalkToMineralPF(LocalWorker* lw);
  void doWalkToMineralFinish(LocalWorker* lw);
  void doMine(LocalWorker* lw);
  void doAssignControlCenterAproach(LocalWorker* lw);
  void doWalkToControlCenterPF(LocalWorker* lw);
  void doWalkToControlCenterFinish(LocalWorker* lw);
  void doRerurnRecources(LocalWorker* lw);
  void doWalkToRandomPosition(LocalWorker* lw);
 
  void moveWorker(LocalWorker* lw, sint4 x, sint4 y);

  Random rand; // local RNG
  
};

#endif

