// $Id: FileArchive.C,v 1.1 2006/05/27 17:22:36 orts_furtak Exp $

// This is an ORTS file
// (c) Tim Furtak
// licensed under the GPL

#include "FileArchive.H"
#include "Compress.H"
#include "Global.H"

using namespace std;

map<string, string> FileArchive::name2file;

//===================================================================

void FileArchive::read_archive(const string &filename)
{
  sint4 length;
  string data;

  {
    ifstream file(filename.c_str(), ios_base::in | ios_base::binary);
    if (!file.is_open()) ERR2("can't open file: ", filename.c_str());
    
    file.seekg(0, ios::end);
    length = file.tellg();
    file.seekg(0, ios::beg);
    
    // read file into buffer
    char *buffer = new char[length];
    file.read(buffer, length);
    file.close();

    string temp(buffer, length);
    delete [] buffer;

    Decompress d;
    d.doit(temp, data);
  }
  
  const char *ptr = data.c_str();
  
  sint4 num_files;
  memcpy(&num_files, ptr, sizeof(num_files));
  ptr += sizeof(num_files);

  Vector<string> files;

  FORU (i, num_files) {
    const char *end = ptr;
    while (*end) ++end;
    string fname(ptr, end-ptr);
    ptr = end+1;
    files.push_back(fname);
  }

  FORU (i, num_files) {
    sint4 size;
    memcpy(&size, ptr, sizeof(size));
    ptr += sizeof(size);

    string data(ptr, size);
    ptr += size;

    name2file[files[i]] = data;
  }
}

//-------------------------------------------------------------------

void FileArchive::make_archive(const string &filename, const Vector<string> &files,
                               sint4 compression_level)
{
  sint4 nf = files.size();

  string num_files((char*)&nf, sizeof(nf));
  string file_names;
  string content;

  FORU (i, files.size()) {
    file_names += files[i];
    file_names += char(0);

    ifstream file(files[i].c_str(), ios::in | ios::binary);
    if (!file.is_open()) ERR2("can't open file: ", files[i].c_str());
    
    // read file into buffer
    sint4 length;
    char *buffer;
    
    file.seekg(0, ios::end);
    length = file.tellg();
    file.seekg(0, ios::beg);
    
    buffer = new char[length];
    file.read(buffer, length);
    file.close();

    string d((char*)&length, sizeof(length));
    d += string(buffer, length);

    content += d;

    delete [] buffer;
  }

  string arch = num_files + file_names + content;
  string archive;

  Compress c(compression_level);
  c.doit(arch, archive);

  ofstream out;
  out.open(filename.c_str(), ios::binary);
  out.write(archive.c_str(), archive.size());
  out.close();
}

//-------------------------------------------------------------------

const bool FileArchive::read_file(const string &filename, string &data)
{
  map<string, string>::iterator i = name2file.find(filename);
  if (i != name2file.end()) {
    data = i->second;
    return true;
  }


  ifstream file;
  file.open(filename.c_str(), ios::binary);
  if (!file.is_open()) {
    REM2("can't open file: ", filename.c_str());
    return false;
  }

  // read file into buffer
  sint4 length;
  char *buffer;

  file.seekg(0, ios::end);
  length = file.tellg();
  file.seekg(0, ios::beg);

  buffer = new char[length];
  file.read(buffer, length);
  file.close();

  data = string(buffer, length);
  delete [] buffer;

  return true;
}

//===================================================================

