#ifndef Octree_H
#define Octree_H

// $Id: Octree.H,v 1.1 2006/06/27 02:32:30 orts_mburo Exp $
 
// (c) Michael Buro, licensed under the GNU Public License, version 2

#include "GfxGlobal.H"
#include "Frustum.H"
#include "Vector.H"

class GfxObject;
class Frustum;

// container of intersecting objects

struct Sector {
  bool used;
  Vector<GfxObject*> objs;
  
  Sector() { used = false; }
};


class Octree {

public:

  enum { NO_CULLING, OCTREE, CONE };

  sint4 culling_mode;

  sint4 add_count;
  sint4 remove_count;
  
  // initialize tree with world bounding box
  Octree(const Coor3 &orig, const Coor3 &dim, sint4 levels=6);
  ~Octree();

  sint4 used() const;
  
  void update(GfxObject *gob);

  // add to object to tree (tree is not owner!)
  void add(GfxObject *gob);

  // remove object from tree and update path to root
  void remove(GfxObject *gob);
  
  // return objects within current OpenGL frustum
  void draw(Vector<GfxObject*> &to_be_drawn);

  Vector<GfxObject*> all_objs;

protected:

  Vector<Sector*> sectors;
  Vector<sint4> sectors_n;

  sint4 sec_levels; // number of octree levels
  Coor3 world_orig, world_dim; // world axis aligned bounding box

  Frustum fr;

  uint4 drawn_magic;

  void index_range(real4 x1, real4 x2, real4 xmin, real4 xmax, sint4 nsec,
		   sint4 &xi_min, sint4 &xi_max);

  // collect all objects in view in vector, set pick value in models to pick
  void draw_scene(const Coor3 &o, const Coor3 &d, sint4 level, sint4 xi, sint4 yi, sint4 zi,
		  Vector<GfxObject*> &to_be_drawn);

  void draw_culled(Vector<GfxObject*> &to_be_drawn);
  void draw_all(Vector<GfxObject*> &to_be_drawn);
};

#endif
