/**
 * $Id: edgewidth.h,v 1.1 2006/12/31 04:09:40 orts_furtak Exp $
 *
 * HOG File
 *
 * Lookup table for the edge widths.
 *
 * The edges are as follows:
 *
 *  ---------
 *  |   |   |
 *  ---------
 *  |  -|-  |
 *  ---------
 *
 * To cross the bottom middle edge.
 *
 * Tile types:
 *   Empty: 0
 *   Forward slash: 1
 *   Backslash: 2
 *
 *
 * To translate into the lookup table index, multiply
 * tile 1's type by 27, multiply tile 2's type by 9,
 * multiply tile 3's type by 3, and add all the values together,
 * along with tile 4.
 *
 * So you end up with: 27*T1 + 9*T2 + 3*T3 + T4
 *
 * For example,
 *
 *  ---------
 *  | \ | / |
 *  ---------
 *  |   | \ |
 *  ---------
 *
 * 27*2 + 9*1 + 3*0 + 2 = 65
 *
 * The edge width will be found in index 65.
 *
 */

#ifndef EDGEWIDTH_H
#define EDGEWIDTH_H
 
static const double ONE = 1.0f;
static const double TWO = 2.0f;
static const double ROOT_TWO = 1.414213562f;
static const double ONE_OVER_ROOT_TWO = 0.707106781f;
 
 
extern "C" {
 
  float edgewidth[81] = {
    TWO,
    ROOT_TWO,
    ONE,
    ONE,
    ONE,
		
    ONE_OVER_ROOT_TWO,
    ROOT_TWO,
    ROOT_TWO,
    ONE,
    ONE,
		
    ONE,
    ONE,
    ONE,
    ONE,
    ONE_OVER_ROOT_TWO,
		
    ONE,
    ONE,
    ONE,
    ROOT_TWO,
    ROOT_TWO,
		
    ONE,
    ONE,
    ONE,
    ONE_OVER_ROOT_TWO,
    ROOT_TWO,
		
    ROOT_TWO,
    ONE,
    ROOT_TWO,
    ROOT_TWO,
    ONE,
		
    ONE,
    ONE,
    ONE_OVER_ROOT_TWO,
    ROOT_TWO,
    ROOT_TWO,
		
    ONE,
    ONE,
    ONE,
    ONE,
    ONE,
		
    ONE,
    ONE_OVER_ROOT_TWO,
    ONE,
    ONE_OVER_ROOT_TWO,
    ONE,
		
    ROOT_TWO,
    ROOT_TWO,
    ONE,
    ONE,
    ONE,
		
    ONE_OVER_ROOT_TWO,
    ROOT_TWO,
    ROOT_TWO,
    ONE,
    ONE,
		
    ONE,
    ONE,
    ONE,
    ONE,
    ONE_OVER_ROOT_TWO,
		
    ONE,
    ONE,
    ONE,
    ONE,
    ONE,
		
    ONE,
    ONE,
    ONE,
    ONE_OVER_ROOT_TWO,
    ONE,
		
    ONE,
    ONE,
    ONE,
    ONE,
    ONE,
		
    ONE,
    ONE,
    ONE_OVER_ROOT_TWO,
    ONE,
    ONE,
		
    ONE
  };
}


#endif
