#ifndef EventHandler_H
#define EventHandler_H

// $Id: EventHandler.H,v 1.3 2006/05/29 19:34:42 ddeutscher Exp $

// This is an ORTS file (c) Michael Buro, licensed under the GPL

#include "Global.H"

//===================================================================

class EventFactory
{
  static sint4 current_who;
  static sint4 current_what;

public:
  /** @return a unique id */
  static sint4 new_who () { return current_who++; }
  static sint4 new_what() { return current_what++; }

};

//===================================================================

class Event {

public:
  
  Event(sint4 who_, sint4 what_) : who(who_), what(what_) {}
  virtual ~Event() {}
  
  sint4 get_who () const { return who; }
  sint4 get_what() const { return what;}

protected:
  sint4 who;
  sint4 what;
};

//===================================================================

class EventHandler
{
public:

  typedef bool (*Handler)(const Event &event);

  // return true if handled 
  virtual bool handle_event(const Event &event) = 0;
  virtual ~EventHandler() {}
};

REGISTER_TYPEOF(130, Vector<EventHandler*>::iterator);
REGISTER_TYPEOF(131, Vector<EventHandler*>::const_iterator);

//===================================================================

class EventHandlerList
{
public:

  void add(EventHandler *h)
  {
    FIND(list, it, h);
    if (it != list.end()) {
      ERR("Handler already added!");
    }
    list.push_back(h);
  }

  void remove(EventHandler *h)
  {
    FIND(list, it, h);
    if (it == list.end()) {
      ERR("Handler not in list!");
    }
    
    list.erase(it); // list.remove(begin,end,h) DOES NOT WORK AS IT DOES NOT CHANGE list.size()!!!
  }

  sint4 call(const Event &e);

protected:

  Vector<EventHandler*> list;
};

//===================================================================

#endif
