/** @file MapTool.C
    @see MapTool.H

    $Id: MapCTF.C,v 1.1 2006/05/29 05:09:17 orts_furtak Exp $
    $Source: /usr/bodo1/cvs/cvs/orts3/libs/serverclient/src/MapCTF.C,v $
*/
// This is an ORTS file (c) Michael Buro, licensed under the GPL

#include "Global.H"
#include "MapTool.H"
#include "GameConst.H"
#include "Random.H"
#include "Options.H"

using namespace std;

//===================================================================

// defined to simplify the CTF map for debugging - less walls, etc.
//#define CTF_SIMPLE
#define CTF_NO_WALLS

// defines an extra wall - block out the center passageway
//#define EXTRA_WALL

// defined to debug winning condition - only places 1 unit per side
//#define CTF_EASY_WIN

//#define OMIT_FLAG_AND_POST

// for debugging - flip top/bottom sides
//#define CTF_FLIP_SIDES

bool CREATE_WATER = false;

//===================================================================

/** Initialize the world with a capture the flag map. */

void MapTool::generate_ctf_map(stringstream &world, bool test_mode)
{
  // Create a fixed, simple capture-the-flag world

  const sint4 num_players = 2;  // always 2 players
  
  sint4 tiles_x, tiles_y;
  sint4 gtiles_x, gtiles_y;

  // use a fixed width and height
  gtiles_x = 20;
  gtiles_y = 20;

  sint4 z_sight = 0;

  const bool rotate = true;

  if (rotate) {
    gtiles_x = sint4(gtiles_x * 0.70710678);
    gtiles_y = sint4(gtiles_y * 0.70710678);
    tiles_x = gtiles_x + gtiles_y;
    tiles_y = gtiles_x + gtiles_y;
  } else {
    tiles_x = gtiles_x;
    tiles_y = gtiles_y;
  }

  world << num_players << endl;
  world << gtiles_x << endl;
  world << gtiles_y << endl;
  world << z_sight << endl;
  if (rotate) world << 1 << endl;
  else        world << 0 << endl;

  sint4 tile_n = tiles_x * tiles_y;

  Map<Tile> map_data;
  map_data.allocate(tiles_x, tiles_y);

  sint2 *height = new sint2[tile_n];
  sint2 *ramp_d = new sint2[tile_n];
  if (!height || !ramp_d) ERR("out of memory");
  memset(height, 0, tile_n*sizeof(*height));
  memset(ramp_d, 0, tile_n*sizeof(*ramp_d));

#ifndef CTF_NO_WALLS
  
  // set plateaus to be used as walls, since there are no ramps
  FORS (i, tile_n) {
    sint4 x = i % tiles_x;
    sint4 y = i / tiles_x;

#ifndef CTF_SIMPLE
    // define horizontal walls
    if ((y == 1) && (x >= 12) && (x < 16)) height[i] = 1;
    if ((y == 7) && (x >= 10) && (x < 17)) height[i] = 1;
    if ((y == 10) && (x >= 10) && (x < 17)) height[i] = 1;
      
    if ((y == 12) && (x >= 7) && (x < 12)) height[i] = 1;
      
#ifdef EXTRA_WALL
    if ((y == 13) && (x >= 12) && (x < 15)) height[i] = 1;
    if ((y == 14) && (x >= 12) && (x < 15)) height[i] = 1;
#endif
    
    if ((y == 15) && (x >= 15) && (x < 21)) height[i] = 1;
      
    if ((y == 17) && (x >= 10) && (x < 17)) height[i] = 1;
    if ((y == 20) && (x >= 10) && (x < 17)) height[i] = 1;
    if ((y == 26) && (x >= 12) && (x < 16)) height[i] = 1;
	
    // define vertical walls
    
    if ((x == 1) && (y >= 12) && (y < 16)) height[i] = 1;
    if ((x == 6) && (y >= 11) && (y < 17)) height[i] = 1;
      
    if ((x == 11) && (y >= 10) && (y < 15)) height[i] = 1;
    if ((x == 15) && (y >= 13) && (y < 18)) height[i] = 1;
      
    if ((x == 21) && (y >= 11) && (y < 17)) height[i] = 1;
    if ((x == 26) && (y >= 12) && (y < 16)) height[i] = 1;

#else
    // minimal walls
    if ((y == 1) && (x >= 12) && (x < 16)) height[i] = 1;
    if ((y == 10) && (x >= 12) && (x < 15)) height[i] = 1;
    if ((y == 17) && (x >= 12) && (x < 15)) height[i] = 1;
    if ((y == 26) && (x >= 12) && (x < 16)) height[i] = 1;
  
    if ((x == 1) && (y >= 12) && (y < 16)) height[i] = 1;
    if ((x == 26) && (y >= 12) && (y < 16)) height[i] = 1;
#endif

  }

#endif //CTF_NO_WALLS
  
  // mark tiles that aren't part of the board
  if (rotate) {
    FORS (i, tile_n) {
      sint4 x = i % tiles_x;
      sint4 y = i / tiles_x;

      if ((x + y < gtiles_y-1) ||
          (tiles_x - x + tiles_y - y < gtiles_y+1) ||
          (tiles_x - x + y < gtiles_x) ||
          (x + tiles_y - y < gtiles_x))
        { height[i] = -2; continue; }
    }
  }

  FORS (i, tile_n) {

    if (height[i] >= 0) {
      map_data(i).set_type(GROUND);

      if (ramp_d[i]) {
	if (ramp_d[i] <= 8) {
	  map_data(i).set_ramp(height[i], height[i]+1,
			       static_cast<Tile::Compass>(ramp_d[i]));
	} else {
	  map_data(i).set_ramp(height[i], height[i]+1, height[i]+1,
			       static_cast<Tile::Compass>(ramp_d[i]-10));
	}
      } else {
        map_data(i).set_flat(height[i]);
      }

    } else {
      map_data(i).set_type(Tile::UNKNOWN);
    }
  }

  world << "<TILES>" << endl;
  map_data.write(world);
  world << "</TILES>" << endl;

  string fname = "data/ctf_units.bp";  // only supported blueprint file
  //fname = opt.bp_file;
  fname = sanitize_path(fname);
  parse_map_include(world, fname);

  
  world << "<OBJECTS>\n";

  // place player 0 units

#ifdef CTF_FLIP_SIDES
  const sint4 side0 = 1;
  const sint4 side1 = 0;
#else
  const sint4 side0 = 0;
  const sint4 side1 = 1;
#endif //CTF_FLIP_SIDES
  
#ifndef OMIT_FLAG_AND_POST
  place_unit(world, "ctf_post", side0, 13, 3);
  place_unit(world, "ctf_flag", side0, 14, 3);
#endif
  
#ifndef CTF_EASY_WIN
  FORS (i, 5) {
    place_unit(world, "ctf_unit", side0, 11 + i, 5);
  }
#else
  // one unit, easy win!
  place_unit(world, "ctf_unit", side0, 11, 5);
#endif //CTF_EASY_WIN
  
  // place player 1 units

#ifndef OMIT_FLAG_AND_POST
  place_unit(world, "ctf_post", side1, 13, 24);
  place_unit(world, "ctf_flag", side1, 14, 24);
#endif

  // if in test mode, place no units (pure CTF time test)
  if (!test_mode) {  
#ifndef CTF_EASY_WIN
    FORS (i, 5) {
      place_unit(world, "ctf_unit", side1, 11 + i, 22);
    }
#else
    // one unit, easy win!
    place_unit(world, "ctf_unit", side1, 11, 22);
#endif //CTF_EASY_WIN
  }
  
  world << "</OBJECTS>\n";

  delete [] height;
  delete [] ramp_d;
}

//===================================================================
