/*
    (c) Michael Buro, mic@research.nj.nec.com
    NEC Research Institute
    4 Independence Way
    Princeton, NJ 08540, USA

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "RT_GameParam.H"

bool RT_GameParam::parse(ostream &oserr, istream &is)
{
  is >> type;
  if (!is) goto error;
 
  x = y = 600;
  n = 2;
  s = 3;
 
  if (type == "L") {

    type = "reg";
    x = y = 800;
    n = 2;
    s = 2;
    
  } else if (type == "S") {

    type = "reg";
    x = y = 50;
    n = 2;
    s = 4;

  } else if (type == "R") {

    type = "reg";
    x = y = 600;
    n = 2;
    s = 3;

  }

  // scan for variables

  FOREVER {

    char c, c2;
    sint4 v;
    
    is >> c;
    if (is.eof()) return false;
    is >> c2;
    is >> v;
    if (!is) goto error;

    switch (c) {
    case 'x':
      x = v;
      if (x < 1 || x > 1600) { oserr << "x<1 | x>1600" << endl; return true; }
      break;
    case 'y':
      y = v;
      if (y < 1 || y > 1600) { oserr << "y<1 || y>1600" << endl; return true; }
      break;
    case 'n':
      n = v;
      if (n < 1) { oserr << "n < 1" << endl; return true; }
      break;
    case 's':
      s = v;
      if (s < 1 || s > 30) {
	oserr << "s < 1 | s > 30" << endl;
	return true;
      }
      break;

    default:

    error:
      
      oserr <<
"illegal game parameters! format: \
(S|R|L|type) (x=<int>|y=<int>|p=<int>|s=<int>)*" << endl;
      return true;
    }
  }
}


void RT_GameParam::write(ostream &os)
{
  os << type << " x=" << x << " y=" << y << " n=" << n << " s=" << s;
}
