/*
    (c) Michael Buro, mic@research.nj.nec.com
    NEC Research Institute
    4 Independence Way
    Princeton, NJ 08540, USA

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef SocketBuffer_H
#define SocketBuffer_H

#include "Global.H"

// basic buffered socket I/O, used by both server and client
// fixme: messy. needs a new design

class Server;
class ServerClient;

class SocketBuffer {

public:
  
  sint4 my_socket;

  sint4 ibufflen;
  sint1 *ibuff;       // linear buffer (contents is moved)
  sint4 ipos, ilen;

  sint4 obufflen;
  sint1 *obuff;       // cyclic buffer (capacity BUFFLEN-1)
  sint4 ostart, oend; // ostart==oend -> buffer empty

  typedef uint1 MsgLenType;
  enum { NO_MSG_LEN = 255 };
  MsgLenType msg_len;

  void init(sint4 ibl, sint4 obl) {
    if (ibl <= 1 || obl <= 1) ERR("bufflen <= 1");
    ibufflen = ibl;
    obufflen = obl;
    my_socket = NO_SOCKET;
    ilen = ipos = 0;
    ostart = oend = 0;
    ibuff = new sint1[ibufflen];
    obuff = new sint1[obufflen];
    msg_len = NO_MSG_LEN;
  }

public:

  static const sint4 NO_SOCKET = -1;

  SocketBuffer(sint4 ibl=32768, sint4 obl=32768) { init(ibl, obl); }
  
  virtual ~SocketBuffer() {
    delete [] obuff;
    delete [] ibuff;
  }

  void set_socket(sint4 so) { my_socket = so; }
  sint4 get_socket() const { return my_socket; }
  
  // # of available bytes in obuff
  sint4 o_available() const {
    if (oend >= ostart) return obufflen - (oend-ostart) - 1;
    return ostart - oend - 1;
  }

  // # of used bytes in obuff,  o_used()+o_available() = obufflen-1
  sint4 o_used() const {
    return obufflen - 1 - o_available();
  }

  sint4 i_used() const { return ilen; }
  
  // <0 : error, 0 : no progress, >0: progress
  sint4 read_socket(bool blocking=true);
  sint4 read(char *msg, sint4 len, bool blocking=true);          // read from buffer
  sint4 write_socket(bool blocking=true);
  sint4 write(const char *msg, sint4 len, bool blocking=true);   // write into buffer
  sint4 blocking_flush();                    // block until write buffer is written

  sint4 blocking_write(const char *msg, sint4 len); // write directly to socket, blocking
  
  // reading and writing of arbitrary value data
  template <class T> sint4 write(T &x, bool blocking=true) {
    return write((char*)&x, sint4(sizeof(T)));
  }

  template <class T> sint4 read(T &x, bool blocking=true) {
    return read((char*)&x, sint4(sizeof(T)));
  }

  // <0 <=> error
  sint4 write_msg(const string &msg, bool blocking=true);
  sint4 read_msg(string &msg, bool blocking=true);


  // these routines assume select has been called before
  // call with ready==true right after select, false thereafter
  sint4 read_socket2(bool ready);
  sint4 read2(char *msg, sint4 len, bool ready);
  sint4 read_msg2(string &msg, bool ready);

  friend class Server;
  friend class ServerClient;  
};

#endif
