/*
    (c) Michael Buro, mic@research.nj.nec.com
    NEC Research Institute
    4 Independence Way
    Princeton, NJ 08540, USA

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
  $Header: /usr/bodo1/cvs/cvs/orts3/misc/orts1/String.C,v 1.1.1.1 2004/10/11 09:09:18 orts_mburo Exp $
  $Log: String.C,v $
  Revision 1.1.1.1  2004/10/11 09:09:18  orts_mburo
  foo

  Revision 1.1.1.1  2003/06/11 22:04:54  orts_mburo
  import

  Revision 1.2  2001/08/17 14:58:10  mic


  changed headers

*/

#include "String.H"

void String::parse( const String& Text, String& Name, String& Rest )
{
  Name.erase();
  Rest.erase();

  String::const_iterator it1 = Text.begin();
  String::const_iterator it2;
  while ( it1 != Text.end() &&  isspace(*it1) ) ++it1; // position at the first non-space
  it2 = it1;
  while ( it2 != Text.end() && !isspace(*it2) ) ++it2; // position at the next space
  for ( ; it1 != it2; ++it1 ) Name += *it1;            // copy to Name

  it1 = it2;
  while ( it1 != Text.end() &&  isspace(*it1) ) ++it1; // position at the first non-space
  it2 = Text.end();
  if ( it1 == it2 ) return;
  --it2;
  while ( it2 != it1 && isspace(*it2) ) --it2;         // position at the last non-space
  ++it2;
  for ( ; it1 != it2; ++it1 ) Rest += *it1;            // copy to Rest 
}

void String::parse( const String& Text, vector<String>& Names )
{
  String rest( Text );
  String name;
  String tmp;

  Names.clear();  

  for ( ;; ) {
    String::parse( rest, name, tmp ); rest = tmp;
    if ( name.empty() ) break;
    Names.push_back( name );
  }
}


void String::parse( const String& Text, String& Name, String& Rest, char Sep )
{
  Name.erase();
  Rest.erase();

  String::const_iterator it1 = Text.begin();
  String::const_iterator it2 = it1;
  while ( it2 != Text.end() && *it2 != Sep ) ++it2;    // position at the first Sep
  for ( ; it1 != it2; ++it1 ) Name += *it1;            // copy to Name

  if ( it1 != Text.end() ) ++it1;
  it2 = Text.end();
  for ( ; it1 != it2; ++it1 ) Rest += *it1;            // copy to Rest 
}

void String::parse( const String& Text, vector<String>& Names, char Sep )
{
  String rest( Text );
  String name;
  String tmp;

  Names.erase( Names.begin(), Names.end() );

  for ( ;; ) {
    String::parse( rest, name, tmp, Sep ); rest = tmp;
    if ( name.empty() ) { if ( rest.empty() ) break; else continue; }
    Names.push_back( name );
  }
}

String& String::pack()
{
  {
    String::iterator it = begin();
    String::iterator hi = end();
    for ( ; it != hi && isspace(*it); ++it );
    erase( begin(), it );
  }
  {
    String::reverse_iterator it = rbegin();
    String::reverse_iterator hi = rend();
    for ( ; it != hi && isspace(*it); ++it );
    erase(size() - (it - rbegin()), it - rbegin() );
  }
  
  return *this;
}

void String::form(const char* fmt, ... )
{
  ostringstream oss;
  
  va_list ap;
  va_start(ap, fmt);
  ::form(oss, fmt, ap);
  va_end(ap);

  erase(); append(oss.str());
}

void String::replace(char c1, char c2) {
  String::iterator it = begin();
  String::iterator hi = end();

  for (; it != hi; ++it) if (*it == c1) *it = c2;
}
