// $Id: ctf_ai_main.C,v 1.1.1.1 2007/04/09 19:05:43 pjungkun Exp $

// This is an ORTS file (c) Michael Chung, licensed under the GPL

#include "Global.H"
#include "gfxclient_init.H"
#include "AI_Client.H"
#include "GUI.H"
#include "Options.H"
#include "SDLinit.H"
#include "ShutDown.H"

using namespace std;

// static/global variables
const char *CLIENT_VERSION = "ORTS AI, version 0.6";

#define DEFAULT_AI "mc"
//#define DEFAULT_AI "all"
//#define DEFAULT_AI "random"
//#define DEFAULT_AI "flag"
//#define DEFAULT_AI "explore"

#define DEFAULT_EVAL "normal"

static void add_options(Options &o) {

  o.put("-debug", false, "display debugging info");
  o.put("-timing", false, "display timing info");
  
  o.put("-ai", 
    string(DEFAULT_AI), 
    "AI type: all|mc|random|stand|flag|explore|randsim");
  
  o.put("-eval", string(DEFAULT_EVAL), "Eval type: normal|random|mix");
  
  o.put("-evalmode", string("avg"), "Eval mode: min|avg|max");
  
  o.put("-mcruns", sint4(10), "monte-carlo: number of runs per eval.");
  o.put("-mcsteps", sint4(10), "monte-carlo: length of each run");
  
  o.put("-rmruns", sint4(100), "random-max: number of runs per eval.");
  o.put("-rmdist", sint4(10), "random-max: max-dist of each run");
  
  o.put("-replan", true, "have AI replan more often");
  
  //o.put("-dvisi", false, "show visible region");
  //o.put("-dknow", false, "show known tiles");
  //o.put("-dseen", false, "flag objects seen by others");
  //o.put("-dgrid", false, "show the grid");
}


int main(int argc, char **argv)
{
  Options opt;
  
  signals_shut_down(true);

  // handle command line arguments
  gfxclient_add_options(opt);
  add_options(opt);
  if (opt.process(argc, argv, cerr, CLIENT_VERSION)) exit(20);

  // initialize the random number generator
  gfxclient_init_rnd(opt);

  cout << "first rand number=" << Rand.rand_uint4() << endl;
  
  bool disp;
  opt.get("-disp", disp);

  if (disp) {
    SDLinit::video_init();
  }
  
  SDLinit::network_init();
  
  // initialize and run the AI client
  AI_Client client(opt);
  client.connect();
  
  return 0;
}
